/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* demodqam Scicos demodulator QAM block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 5 janvier 2005 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <scicos/scicos_block.h>
#include "modnum_lib.h"

/* Cette fonction de simulation ralise un decodage QAM dfini par la fonction
 * modqam.c. Les signaux I et Q reu en entre sont converti aux valeurs des
 * bits de poids fort et des bits de poids faibles du numro symbole.
 * Les valeurs converties sont ensuite additionnes et la somme est place
 * dans le registre de sortie y[].
 *
 * Entres rgulires : u1[0..insz[0]-1]: vecteur des signaux I
 *                      u2[0..insz[0]-1]: vecteur des signaux Q
 * Sortie rgulire :   u[0..insz[0]-1] : vecteur des numros symboles
 * Entre vnementielle : instants de dcision
 * Sortie vnementielle : nant
 * Paramtres entiers : ipar[0..insz[0]-1] : vecteur des nombres de bits.
 *
 * nb : ne fonctionne que pour des modulations avec un nombre de valeurs
 *     identiques pour chaque axes I et Q(16QAM, 64QAM, 256QAM, ...)
 */

/*prototype*/
void demodqam(scicos_block *block,int flag)
{
 /*dclaration des variables*/
 double *y;
 double *u1,*u2;
 int nu;

 /*Rcupration des adresses des ports rguliers*/
 y=(double *)block->outptr[0];
 u1=(double *)block->inptr[0];
 u2=(double *)block->inptr[1];

 /*rcupration de la taille des ports d'entres*/
 nu=block->insz[0];

 /*Le flag 1 calcule le numro symbole*/
 if(flag==1)
 {
  /*Appel demoqamv_c*/
  demodqamv_c(&nu,&block->ipar[0],&u1[0],&u2[0],&y[0]);
 }
}
