/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* modpsk Scicos Mary Phase Shift Keying modulator block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 21 dcembre 2004 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <scicos/scicos_block.h>
#include "modnum_lib.h"

/* Cette fonction de simulation ralise un codeur MPSK.
 * Les sorties y1 et y2 correspondent aux valeurs de I et de Q
 * codes en fonction du nombre d'tats et du numro symbole
 * prsent  l'entre u[] aux dates de dclanchements.
 * La fonction calcule :
 * I=cos(%pi*(2*u+1)/m) 
 * Q=-sin(%pi*(2*u+1)/m)
 * o u est l'entre et m le nombre d'tats.
 * 
 * entres rgulires      : u[0..insz[0]-1]  : vecteur des numros symboles
 * sorties rgulires      : y1[0..insz[0]-1] : vecteur des composantes I
 *                           y2[0..insz[0]-1] : vecteur des composantes Q
 * entre vnementielles  : instants de dclenchement.
 * sortie vnementielle   : nant
 * paramtres entiers      : ipar[0..insz[0]-1]:m vecteur des nombre d'tats
 */

/*prototype*/
void modpsk(scicos_block *block,int flag)
{
 /*Dclaration des variables*/
 double *y1,*y2;
 double *u;

 /*Rcupration des adresses des ports rguliers*/
 y1=(double *)block->outptr[0];
 y2=(double *)block->outptr[1];
 u=(double *)block->inptr[0];

 /*
 Le flag 1 calcule la valeur de I et de Q en fonction
 du numro symbole u[] et du nombre du nombre d'tat m
 */
 if(flag==1||flag==6)
 {
  /*Appel routine modpsk*/
  modpskv_c(&block->insz[0],&block->ipar[0],&u[0],&y1[0],&y2[0]);
 }
}
