/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* imodulo Scicos integer modulo function block
 * Type 2 simulation function - scilab-2.6&2.7&3.0
 * IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "machine.h"

/* Cette fonction de simulation propose de raliser la fonction
 * modulo rencontre dans les systmes numriques traitant les oprations
 * sur entier en code complment  2.
 * entres rgulires : u[0..nu-1]
 * sorties rgulires : y[0..nu-1] = mod[u[0..nu-1]]
 * paramtres entiers : ipar[0..nu-1] : nombre de bits des mots entiers.
 *
 *Rmq : l'entre de type double est tronque  sa valeur entire.
 *          la sortie est soit ngative soit positive
 */

/*Prototype*/
void
imodulo(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
        ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
    /*Dclaration des variables*/
    int i,nu;
    double *y;
    double *u;
    long ent; /*Dclaration d'un entier de type long*/

    /*Rcupration des adresses des ports rguliers*/
    y=(double *)outptr[0];
    u=(double *)inptr[0];

    /*Rcupration de la taille du port d'entre*/
    nu=insz[0];

    for (i=0;i<nu;i++)
    {
     /*Rcupration de la valeur d'entre*/
     ent = (long) u[i];

     /*Ralisation du tronquage code complment  2*/
     ent -= 2<<(ipar[i]-2);
     ent &= (2<<(ipar[i]-1)) - 1;
     ent -= 2<<(ipar[i]-2);

     /*Place valeur tronque dans le registre de sortie*/
     y[i] = ent;
   }
}

