/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* logistique Scicos logistique function block
 * Type 2 simulation function ver 1.0 - scilab-2.6&2.7
 * 18 novembre 2003 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "machine.h"

/* Cette fonction de simulation ralise la fonction non linaire logistique :
 * y = 4*R*u*(1-u)
 * o R est un paramtre, y la sortie et u l'entre. 
 *
 * Entres rgulires : u1[0..nu-1] : vecteur du paramtre R
 *                      u2[0..nu-1] : vecteur d'entre
 * Sortie rgulire : y1[0..nu-1] : vecteur des sorties
 * Entre vnementielle : nant (hritage)
 * sortie vnementielle : nant
 * paramtres : nant.
 */

/*prototype*/
void logistique(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
                ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
 /*dclaration des variables*/
 double *y1;
 double *u1;
 double *u2;
 int nu,i;

 /*rcupration des adresses des ports rguliers*/
 y1=(double *)outptr[0];
 u1=(double *)inptr[0];
 u2=(double *)inptr[1];

 /*rcupration de la taille du port d'entre u1*/
 nu = insz[0];

 /*Calcule du registre de sortie*/
 for(i=0;i<nu;i++) y1[i]=4*u1[i]*u2[i]*(1-u2[i]);
}
