/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* int_trap Scicos discrete integral by trapezoidal method block
 * Type 2 simulation function ver 1.0 - scilab-2.6&2.7
 * 17 novembre 2003 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "machine.h"

/* Cette fonction de simulation ralise une intgration discrte
 * du vecteur d'entre par la mthode des trapzes :
 * y[k]=h/2*(u[k]+u[k-1])+y[k-1]
 * o h est le pas d'intgration, y[k] le vecteur de sortie,
 * et u[k] le vecteur d'entre
 *
 * entre rgulires : u[0..nu-1] vecteur d'entre
 * sorties rgulires : y[0..nu-1] registre de sortie
 * entre vnementielle : Instants de dclenchement
 * sortie vnementielle : nant.
 * tat discret : z[0..nu-1]    vecteur des tats de sortie discrets prcdents
 *                z[nu..2*nu-1] vecteur des tats d'entre discrets prcdents
 *
 * paramtre rel : rpar[0] pas d'intgration
 *
 */

/*prototype*/
void int_trap(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
              ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
  /*dclaration des variables*/
  double *y;
  double *u;
  int i,nu;

  /*rcupration de l'adresses des ports rguliers*/
  y=(double *)outptr[0];
  u=(double *)inptr[0];

  /*Rcupration de la taille du port d'entre*/
  nu=insz[0];

  /*Le flag 1 calcule l'integrale et place le rsultat dans le registre y[]*/
  if(*flag==1)
  {
   for(i=0;i<nu;i++) y[i]=(rpar[0]/2)*(u[i]+z[nu+i])+z[i];
  }

  /*le flag 2 place l'tat de sortie y et l'tat d'entre dans z*/
  else if(*flag==2)
  {
   for(i=0;i<nu;i++)
   {
    z[i]=y[i];
    z[nu+i]=u[i];
   }
  }
}
