/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* stockbit Scicos bit accumulator
 * Type 2 simulation function ver 1.1 - scilab-2.6&2.7
 * 12 novembre 2003 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "machine.h"

/* Cette fonction de simulation ralise un accumulateur de bit
 * Chaque paquet de bit en entre (de longueur ipar[0..nu-1])
 * est stock dans un mot de longueur ipar[0..nu-1]*ipar[nu..2*nu-1]
 * o ipar[nu..2*nu-1] reprsente le nombre de paquets  stocker.
 * Le ime mot est plac  la place i*ipar[0..nu-1] et le mot
 * de sortie de longueur ipar[nu..2*nu-1] est cod en code complment  2
 * 
 * Entre rgulire : u[0..nu-1] vecteur des entiers cods sur Nbit
 * Sortie rgulire : y[0..nu-1] vecteur des entiers cods sur Nbit*Nbpaquet
 * Entre vnementielle : Instant d'acquisition du mot en entre
 * Sortie vnementielle : nant
 * Paramtres entiers : ipar[0..nu-1] : longueur des paquets du vecteur d'entre (Nbit)
 *                      ipar[nu..2*nu-1] : Nombre de paquet  stocker
 * Mmoires : z[0..nu-1] : compteur paquet
 *            z[nu..2*nu-1] : valeur du mot de sortie.
 */
 
/*prototype*/
void stockbit(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
              ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
  /*dclaration des variables*/
  int i,j,k,nu;
  double *y;
  double *u;

  /*Rcupration des adresses des ports rguliers*/
  y=(double *)outptr[0];
  u=(double *)inptr[0];

  /*rcupration de la taille du port d'entre*/
  nu=insz[0];

  if(*flag==1)
  {
   for(k=0;k<nu;k++)
   {
    /*Incrment compteur paquet*/
    z[k]++;

    /*Acquisition de l'entre dans i*/
    i = (int)u[k];
    /*fprintf(stderr,"stockbit z[k]=%f, u=%f\n", z[k],u[k]);*/

    /*Conversion en nombre entier non sign*/
    i &= (2<<(ipar[k]-1)) - 1;

    /*Dcale  gauche le paquet de z[k]*ipar[k] bits*/
    i=i<<(ipar[k]*(int)(z[k]-1));

    /*Incrmente valeur paquet*/
    z[nu+k] = z[nu+k]+i;

    /*Ralise test sur compteur paquet*/
    if (z[k]==ipar[nu+k])
    {
     /*Passe valeur de z[nu+k] dans j*/
     j=(int)z[nu+k];

     /*Ralise conversion code complment  2*/
     j-= 2<<((ipar[k]*ipar[nu+k])-2);
     j&= (2<<((ipar[k]*ipar[nu+k])-1)) - 1;
     j-= 2<<((ipar[k]*ipar[nu+k])-2);

     /*Place la valeur du paquet dans registre y[]*/
     y[k]=j;
     /*fprintf(stderr,"stockbit y=%f\n", y[k]);*/
     /*Met la valeur du compteur paquet  zro*/
     z[k]=0;

     /*Rinitialise la valeur du paquet de sortie  zro*/
     z[nu+k]=0;
    }
   }
  }
}
