/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* cpf Scicos Phase/frequency Comparator block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 6 janvier 2005 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"
#include "scicos_block.h"

/* Cette fonction de simulation ralise un comparateur phase frquence
 * trois tats. Elle calcule les signaux discrets Up(t) et Down(t) qui
 * sont les sorties logiques de deux bascules D. Les entres de ces bascules
 * sont places  1 et les entres d'horloges de celles-ci sont les entres
 * vnementielles du bloc. Lorsque les sorties des deux bascules sont toutes
 * les deux places  1, la fonction place les sorties du bloc y[0] et y[1]  0.
 *
 * entres d'venements : dates d'vnements Ref(t), Div(t)
 * sortie d'venement  : nant
 * entre rgulires : nant
 * Sortie rgulires : Up(t) et Down(t)
 * paramtres : nant
 * Etats discrets : z[0] et z[1] mmorisent les tats prcdents des sorties des bascules
 */

/*prototype*/
void cpf(scicos_block *block,int flag)
{
 /*dclaration*/
 int k;
 int nev;
 double *y1, *y2;

 /*rcupration des adresses*/
 y1=(double *)block->outptr[0];
 y2=(double *)block->outptr[1];
 nev=block->nevprt;

 /* Le flag 1 met  jour les sorties un test est ralis sur les tats prcdents*/
 if(flag==1)  {
  /*Appel cpfy_c*/
  cpfy_c((k=1,&k),&nev,&block->z[0],&block->z[1],&y1[0],&y2[0]);
 }
 /* Le flag 2 met  jour les tats prcdents*/
 else if(flag==2)  {
  /*Appel cpfz_c*/
  cpfz_c((k=1,&k),&nev,&block->z[0],&block->z[1]);
 }
}
