/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* pfdt Scicos Phase Frequency Detector block including 
 * Phase/Frequency Comparator with delay element for the RAZ of flip-flop
 * and charpe pump
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 13 janvier 2004 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "scicos_block.h"
#include "stdio.h"

/* entres rgulires      : nant
 * sorties : y[0..ny-1]   : vecteur de la pompe de charge
 *
 * entres vnementielles : nev = 1 : entre d'horloge de mise  jour du cpf
 *                           nev = 2 : entre d'horloge du diviseur de retour
 *                           nev = 4 : entre d'horloge de RAZ
 *
 * sortie vnementielles  : evout[0] : vnement de remise  zro
 *
 * paramtres entiers : outsz[0]    : taille des vecteurs de la pompe de charge (ny)
 *                      ipar[0]     : type de bruit de la pompe de charge (typ_leak)
 *                                   0: pas de fuite
 *                                   1: fuite constante
 *                                   2: fuite bruit normal
 *
 * paramtres rels   : rpar[0]      : retard des Remises A Zro
 *                      rpar[1]      : courant max de la pompe de charge
 *                      rpar[2]      : courant de fuite moyen de la pompe de charge
 *                      rpar[3]      : deviation du courant de fuite
 *
 * Etats discrets     : z[0],z[1]  : etats mmoires des bascules
 *                      z[2],z[3]  : etats de sorties des bascules
 *                      z[4]       : etat du comparateur (0:activ,1:dsactiv)
 */
void pfdt_c(scicos_block *block,int flag)
{
 /*dclaration*/
 int nev;
 int sh;
 int i;
 double *y;
 double *z;
 int typ_leak;
 double Io_up,Io_down,Ileak_d,Ileak_m;
 double delay;

 /*rcupration des adresses*/
 y   = (double *)block->outptr[0];
 z   = (double *)block->z;

 nev = block->nevprt;

 /*rcupration des paramtres*/
 typ_leak = block->ipar[0];
 delay    = block->rpar[0];
 Io_up    = block->rpar[1];
 Io_down  = block->rpar[2];
 Ileak_m  = block->rpar[3];
 Ileak_d  = block->rpar[4];

 sh = (int) z[4];

 /* output */
 if(flag==1) {

  switch (nev)
  {
   /* cpf */
   case 1 :
   {
    z[2] = 1;
    break;
   }
   /* div */
   case 2 :
   {
    z[3] = 1;
    break;
   }
   /* raz */
   case 4 :
   {
     z[2]=0;
     z[3]=0;
     /*fprintf(stderr,"nevprt=4, z[0]=%f,z[1]=%f,z[2]=%f,z[3]=%f\n",z[0],z[1],z[2],z[3]);*/
     sh=0;
     break;
   }
   break;
  }

  //z[0]=z[2];
  //z[1]=z[3];

  switch (typ_leak)
  {
   case 0 : /*cas pas de fuite*/
   {
    y[0] = Io_up*z[2]-Io_down*z[3];
    break;
   }
   case 1 : /*cas fuite constante*/
   {
    y[0] = Io_up*z[2]-Io_down*z[3]+Ileak_m;
    break;
   }
   case 2 :
   {
    /*Appel noiseblk_c*/
    noiseblk_c(1,(i=1,&i),&Ileak_d,&Ileak_m,&y[0]);
    y[0] = Io_up*z[2]-Io_down*z[3]+y[0];
    break;
   }
  }

 }

 /* raz */
 else if(flag==3) {

  switch (nev)
  {
   /* cpf */
   case 1 :
   {
    if(z[3] == 1) {
      block->evout[0]=delay;
    }
    break;
   }
   /* div */
   case 2 :
   {
    if(z[2] == 1) {
      block->evout[0]=delay;
    }
    break;
   }
   break;
  }

 }
}
