/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* overlap Scicos temporal overlap vector
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 5 dcembre 2005 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <scicos/scicos_block.h>
#include "modnum_lib.h"

/* Cette fonction transforme un vecteur de taille nu en vecteur de taille ny,
 * avec nu>ny. Elle mmorise l'excs nu-ny et ajoute celui au dbut du vecteur
 * suivant.
 * Entres rgulires : u[0..nu-1] : vecteur d'entre de taille nu
 * sorties rgulires : y[0..ny-1] : vecteur de sortie de taille ny
 * paramtres entiers : nz : taille du vecteur Z
 */

/*prototype*/
void overlap(scicos_block *block,int flag)
{
  /*dclaration*/
  double *y;
  double *u;
  double *z;
  int i,nu,ny,k,l;

  /*rcupration de l'adresse des ports rguliers*/
  y=(double *)block->outptr[0];
  u=(double *)block->inptr[0];
  z=block->z;

  /*Rcupration de la taille des ports rguliers*/
  nu=block->insz[0];
  ny=block->outsz[0];

  /*La flag 1 calcule y*/
  if(flag==1)
  {
   /*Recopie les ny premiers lments de u dans y*/
   copyd_c(&ny,&u[0],&y[0]);

   /*Ajoute les nz lments prcdents au dbut du vecteur y*/
   for(i=0;i<block->nz;i++) y[i]=u[i]+z[i];
  }
  /*Le flag 2 mais en mmoire l'xcs nz de u dans Z*/
  else if(flag==2)
  {
   /*Recopie les nz xcdentaires de u dans z*/
   copyd_c((l=block->nz,&l),&u[ny],&z[0]);
  }
}
