/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* surech Scicos oversampling by fft
 * Type 2 simulation function ver 1.0 - scilab-2.6&2.7&3.0
 * 23 dcembre 2003 - IRCOM GROUP - Author : A.Layec
 */

 /* REVISION HISTORY :
 * $Log$
 */

#include "machine.h"

/* Cette fonction ralise le surchantillonnage par fft. Elle recopie N fois le vecteur
 * d'entre[u1;u2] dans le vecteur de sortie [y1;y2]
 *
 * entres rgulires : u1[0..nu-1] : parties relles du signal frquentiel
 *                      u2[0..nu-1] : parties imaginaires du signal frquenteil
 * sorties rgulires : y1[0..ny-1] : parties relles du signal frquentiel surechantillonn
 *                      y2[0..ny-1] : parties imaginaires du signal frquentiel surechantillonn
 * paramtres entiers : ipar[0] : N facteur de surechantillonnage
 *
 * nb : ny=N*nu
 */

/*prototype*/
void surechfft(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
               ipar,nipar,inptr,insz,nin,outptr,outsz,nout)

int *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
  /*Dclaration des variables*/
  double *y1,*y2;
  double *u1,*u2;
  int i,j,N,nu,ny;

  /*Rcupration des adresses des ports rguliers*/
  y1=(double *)outptr[0];
  y2=(double *)outptr[1];
  u1=(double *)inptr[0];
  u2=(double *)inptr[1];

  /*Rcupration de la taille du port d'entre*/
  nu=insz[0];
  ny=outsz[0];

  /*Rcupration du facteur de surchantillonnage*/
  N=ipar[0];

  /*Rcopie u dans y*/
  for(i=0;i<N;i++)
  {
   for(j=0;j<nu;j++)
   {
    y1[i*nu+j]=u1[j];
    y2[i*nu+j]=u2[j];
   }
  }
}
