/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* z_buf41 Scicos discrete memory block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 31 mars 2007 - INRIA - Author : A.Layec
 *  Rvision
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "machine.h"
#include "scicos_block.h"

#ifdef __STDC__
extern int  Scierror __PARAMS((int iv,char *fmt,...));
#else
extern int Scierror __PARAMS(());
#endif

/* Cette fonction de simulation est un buffer
 * qui mmorise la valeur de son entre rgulire
 * dans le vecteur des tats discrets z
 *
 * entre rgulire : u[0..nu-1]
 * sortie rgulire : nant
 * entre vnementielle : Date de dclenchement
 * sortie vnementielle : nant
 *
 * tat discret : z[0] : compteur
 *                z[1..ipar[0]] : mot mmoire
 *
 * paramtre entier : ipar[0] : taille du mot mmoire
 *                    ipar[1] : avec valeur vnement
 *                              (0: no, 1:yes)
 */

/*prototype*/
void z_buf41(scicos_block *block,int flag)
{
 /*dclaration des variables*/
 int i,j,k;
 int evt;
 int m;

 /*rcupre la taille du buffer*/
 m=block->ipar[0];

 /*rcupre option with_evt*/
 evt = block->ipar[1];

 /*init*/
 if(flag==4)
 {
  /* vrifie que la taille des entres est < 
   * la taille du buffer
   */
  if (block->insz[0]>block->ipar[0]) {
     Scierror(888,"Size of input is greater than the buffer size.\n");
     set_block_error(-3);
     return;
  }
 }

 /*seulement sur flag 2*/
 else if(flag==2)
 {
  /*rcupre valeur du compteur chantillons*/
  j=(int)block->z[0];

  /*Test valeur compteur chantillons*/
  if (j+block->insz[0]<m) {
     if(evt==1) {
       /*met en mmoire la(les) valeur(s) du port rguliers + date*/
       for(i=0;i<block->insz[0];i++) {
         block->z[1+j+i]=get_scicos_time();
       }
       /*pour chaque entre*/
       for(k=0;k<block->nin;k++) {
         for(i=0;i<block->insz[0];i++) {
           block->z[k*m+1+m+j+i]=block->inptr[k][i];
         }
       }
     }
     else {
       /*met en mmoire la(les) valeur(s) du port rguliers*/
       /*pour chaque entre*/
       for(k=0;k<block->nin;k++) {
         for(i=0;i<block->insz[0];i++) {
           block->z[k*m+1+j+i]=block->inptr[k][i];
         }
       }
     }
     /*incrmente compteur chantillons*/
     j=j+block->insz[0];
  }
  else {
     if(evt==1) {
       /*met en mmoire  la fin du buffer + date*/
       for(i=0;i<m-j;i++) {
         block->z[1+j+i]=get_scicos_time();
       }
       /*pour chaque entre*/
       for(k=0;k<block->nin;k++) {
         for(i=0;i<m-j;i++) {
           block->z[k*m+1+m+j+i]=block->inptr[k][i];
         }
       }
       /*met en mmoire au dbut du buffer + date*/
       for(i=m-j;i<block->insz[0];i++) {
         block->z[1+j+i-m]=get_scicos_time();
       }
       /*pour chaque entre*/
       for(k=0;k<block->nin;k++) {
         for(i=m-j;i<block->insz[0];i++) {
           block->z[k*m+1+m+j+i-m]=block->inptr[k][i];
         }
       }
     }
     else {
       /*pour chaque entre*/
       for(k=0;k<block->nin;k++) {
         /*met en mmoire  la fin du buffer*/
         for(i=0;i<m-j;i++) {
           block->z[k*m+1+j+i]=block->inptr[k][i];
         }
         /*met en mmoire au dbut du buffer*/
         for(i=m-j;i<block->insz[0];i++) {
           block->z[k*m+1+j+i-m]=block->inptr[k][i];
         }
       }
     }
    /*met  jour compteur chantillons*/
    j=block->insz[0]-m+j;
  }

  /*mmorise compteur dans z[n+1]*/
  block->z[0]=(double)j;
 }
}
