/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* cpz Scicos tri-state Phase comparator block
 * Type 4 simulation function ver 1.0 - scilab-4.1.2
 * 27 Mars 2009 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "scicos_block4.h"
#define M_PI 3.14159265358979323846

/* Cette fonction de simulation calcul des instants de transistion
 * module en phase.
 * Entres rgulires : phase (compris entre -2*%pi;2*%pi)
 * Sorties rgulires : nant
 * Entres vnementielles : nant
 * Sortie vnementielles :  sortie module
 *
 * paramres : rpar[0] : priode de dclenchement
 *             rpar[1] : priode de dclenchement
 *
 */

typedef struct work_st {
  long long int i;
  long long int j;
} work_struct;

/*prototype*/
void cpz(scicos_block *block,int flag)
{
 /*dclaration*/
 double T;      /*priode fixe*/
 double t;      /*temps simulateur*/
 double *u1;
 double *u2;
 double *y1;
 double *y2;
 int k,nev;

 /*ptr of the struct of that block*/
 work_struct *work;

 /*get he struct of that block*/
 work = (work_struct*) *block->work;

 /*rcupration de l'adresse du port d'entre*/
 u1=(double *)block->inptr[0];
 u2=(double *)block->inptr[1];

 y1=(double *)block->outptr[0];
 y2=(double *)block->outptr[1];

 /*rcupration des paramtres*/
 T      = block->rpar[0]; /* priode */

 /*rcupre valeur temps du simulateur*/
 t = get_scicos_time();

  switch(flag)
  {
   /* initialisation */
   case 4  : /*work est utilis pour stocker les compteurs de priode*/
             if ((work=(work_struct*) scicos_malloc(sizeof(work_struct))) == NULL) {
               set_block_error(-16);
               return;
             }

             /*initialise compteurs*/
             work->i = 1;
             work->j = 1;

             *block->work=(void *)work; /*store my_st in *block->work*/

             break;

   /*le flag 1 calcule les sorties*/
   case 1  : y1[0]=block->z[0];
             y2[0]=block->z[1];

             break;

   /*le flag 2 calcule les tats*/
   case 2  : if (block->nevprt<0) {
               if (block->jroot[0]!=0) {
                 /*augmente compteur */
                 work->i++;
                 /*Appel cpfz_c*/
                 cpfz_c((k=1,&k),(nev=1,&nev),&block->z[0],&block->z[1]);
               }
               if (block->jroot[1]!=0) {
                 /*augmente compteur */
                 work->j++;
                 /*Appel cpfz_c*/
                 cpfz_c((k=1,&k),(nev=2,&nev),&block->z[0],&block->z[1]);
               }
             }

             break;

   /*le flag 9 calcule les surfaces de dtections  zro*/
   case 9  : /*test sur valeur d'entre*/
             if ((u1[0]<(2*M_PI)) && (u1[0]>-(2*M_PI))) {
               block->g[0] = t - ((work->i)*T + T*(u1[0]/(2*M_PI)));
             }
             /*sinon retourne erreur*/
             else {
               /*Coserror("flag=9, valeur de phase incorrecte : %f.\n",u[0]);*/
               /*else*/
                 set_block_error(-1);
               /**/
               return;
             }
             if ((u2[0]<(2*M_PI)) && (u2[0]>-(2*M_PI))) {
               block->g[1] = t - ((work->j)*T + T*(u2[0]/(2*M_PI)));
             }
             /*sinon retourne erreur*/
             else {
               /*Coserror("flag=9, valeur de phase incorrecte : %f.\n",u[0]);*/
               /*else*/
                 set_block_error(-1);
               /**/
               return;
             }

             break;

   /* terminaison */
   case 5  : scicos_free(*block->work); /*libre le workspace*/
             break;

   default : break;
  }

}
