/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* gencw Scicos Constant Wave Generator block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 7 janvier 2005 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <scicos/scicos_block4.h>
#include "modnum_lib.h"

/* Cette fonction dlivre une onde  frquence constante. 
 *
 * entres rgulires : nant
 * sorties rgulires : sortie du gnrateur
 * entre vnementielle : dtermine la cadence de gnration
 *
 * Paramtres rels   : insz[0] : (n) longeur des vecteurs
 *                      insz[1] : (m) nombre de vecteurs
 *                      rpar[0..(m-1)]     : amplitude des ondes de sorties
 *                      rpar[m..(2*m-1)]   : frquences des ondes de sorties
 *                      rpar[2*m..(3*m-1)] : phase  l'origine des ondes de sorties
 *                      rpar[3*m]          : pas d'chantillonnage
 *
 * Paramtres entiers : ipar[0] : option (1:cos,2:sin,3:sin/cos)
 *
 * Etat discret       : valeur courante du temps discret
 *
 */

/*prototype*/
void gencw(scicos_block *block,int flag)
{
 /*dclaration des variables*/
 int i,j;
 int ny;
 int my;
 double *ampl;
 double *fs;
 double *phi;
 double ts;
 double *ti;
 int opt;

 /*declaration ptr scicos*/
 SCSREAL_COP *yd_1;
 SCSREAL_COP *yd_2=NULL;
 SCSREAL_COP *rpar;
 SCSINT_COP  *ipar;
 SCSREAL_COP *dstate;

 /*rcupration de l'adresse de sortie*/
 yd_1=GetRealOutPortPtrs(block,1);

 /*rcupration des tailles des ports de sortie*/
 ny=GetOutPortSize(block,1,1);
 my=GetOutPortSize(block,1,2);

 /*rcupration des paramtres*/
 rpar   = GetRparPtrs(block);
 ipar   = GetIparPtrs(block);
 ampl   = &rpar[0];
 fs     = &rpar[my];
 phi    = &rpar[2*my];
 ts     = rpar[3*my];
 opt    = ipar[0];

 /*rcupration des tats*/
 dstate = GetDstate(block);
 ti     = &dstate[0];

 /*calcul des sorties*/
 if(flag==1) {
  switch (opt)
  {
   case 1 : /*Appel cw_r*/
            cw_c(&ny,&my,ampl,fs,phi,ti,&ts,&opt,yd_1,yd_2);
            break;

   case 2 : /*Appel cw_r*/
            cw_c(&ny,&my,ampl,fs,phi,ti,&ts,&opt,yd_2,yd_1);
            break;

   case 3 : /*rcupration de l'adresse de sortie*/
            yd_2=GetRealOutPortPtrs(block,2);
            /*Appel cw_r*/
            cw_c(&ny,&my,ampl,fs,phi,ti,&ts,&opt,yd_1,yd_2);
            break;
  }
 }
 /*calcul des tats*/
 else if(flag==2) {
   (*ti)=(*ti)+ny*ts;
 }
}
