/* Modnumlib ScicoLab interface routine
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 

/* REVISION HISTORY :
 * $Log$
 */
#include <string.h>
#include <stdio.h> /* to be removed */
#include "stack-c.h"
#include "mex.h"

#include "modnum_lib.h"

extern int C2F(mkmlist)();
extern void str2sci(char** x,int n,int m);
extern int *listentry(int *header, int i);
extern int CopyVarFromlistentry(int lw, int *header, int i);

/*
 * y=genint(n(,m,typ,nbit])
 *
 * Gnrateur de nombres entiers alatoires (int32)
 *
 * Entres : n,m  : dimensions de la matrice de sortie
 *           typ  : type de codage (dfaut 0)
 *                  0 : codage NRZ
 *                  1 : codage RZ ou nombre non sign si nbit>1
 *                  2 : nombre sign si nbit>1
 *           nbit : longueur en nombre de bits des mots
 *                  de sortie (dfaut 1)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgenint_c(char *fname)
{
  static int l1, m1, n1; /*entre 1*/
  static int l2, m2, n2; /*entre 2*/
  static int l3, m3, n3; /*entre 3*/
  static int l4, m4, n4; /*entre 4*/
  int l5;                /*sortie 1*/

  int m=1;

  int *typ_ptr;              /* type de sortie */
  int typ=0;

  int *nbit_ptr;             /* nombre de bits */
  int nbit=1;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(1,4);
  CheckLhs(1,1);

  /* Rcupre les entres et cre la sortie */
  GetRhsVar(1,"i",&m1,&n1,&l1); /* n */
  CheckDims(1,m1,n1,1,1);
  /* verification valeur n */
  if (*istk(l1)<0) {
    Scierror(888,"%s : Number of rows must be positive.\n",fname);
    return 0;
  }

  if (Rhs==1) {
    /* Rcupration nbit/typ */
    typ_ptr  = &typ;
    nbit_ptr = &nbit;

    l5 = I_INT32; /* y */
    CreateVar(2,"I",istk(l1),&m,&l5);

    /*Appel genint_c*/
    geninti_c(istk(l1),&m,nbit_ptr,typ_ptr,istk(l5));

    /* Retourne la variable de sortie */
    LhsVar(1) = 2;

    return 0;
  }
  else if (Rhs==2) {
    GetRhsVar(2,"i",&m2,&n2,&l2); /* m */
    CheckDims(2,m2,n2,1,1);
    /* verification valeur m */
    if (*istk(l2)<0) {
      Scierror(888,"%s : Number of cols must be positive.\n",fname);
      return 0;
    }

    /* Rcupration nbit/typ */
    typ_ptr  = &typ;
    nbit_ptr = &nbit;

    l5 = I_INT32; /* y */
    CreateVar(3,"I",istk(l1),istk(l2),&l5);

    /*Appel genint_c*/
    geninti_c(istk(l1),istk(l2),nbit_ptr,typ_ptr,istk(l5));

    /* Retourne la variable de sortie */
    LhsVar(1) = 3;

    return 0;
  }
  else if (Rhs==3) {
    GetRhsVar(2,"i",&m2,&n2,&l2); /* m */
    CheckDims(2,m2,n2,1,1);
    /* verification valeur m */
    if (*istk(l2)<0) {
      Scierror(888,"%s : Number of cols must be positive.\n",fname);
      return 0;
    }

    /* Rcupration nbit/typ */
    GetRhsVar(3,"i",&m3,&n3,&l3); /* typ */
    CheckDims(3,m3,n3,1,1);
    typ_ptr = istk(l3);
    /* verification valeur typ */
    if ((*typ_ptr!=0)&&(*typ_ptr!=1)&&(*typ_ptr!=2)) {
      Scierror(888,"%s : Type of generator must be 0, 1 or 2.\n",fname);
      return 0;
    }

    nbit_ptr = &nbit;

    l5 = I_INT32; /* y */
    CreateVar(4,"I",istk(l1),istk(l2),&l5);

    /*Appel genint_c*/
    geninti_c(istk(l1),istk(l2),nbit_ptr,typ_ptr,istk(l5));

    /* Retourne la variable de sortie */
    LhsVar(1) = 4;

    return 0;
  }
  else {
    GetRhsVar(2,"i",&m2,&n2,&l2); /* m */
    CheckDims(2,m2,n2,1,1);
    /* verification valeur m */
    if (*istk(l2)<0) {
      Scierror(888,"%s : Number of cols must be positive.\n",fname);
      return 0;
    }

    /* Rcupration nbit/typ */
    GetRhsVar(3,"i",&m3,&n3,&l3); /* typ */
    CheckDims(3,m3,n3,1,1);
    typ_ptr = istk(l3);
    /* verification valeur typ */
    if ((*typ_ptr!=0)&&(*typ_ptr!=1)&&(*typ_ptr!=2)) {
      Scierror(888,"%s : Type of generator must be 0, 1 or 2.\n",fname);
      return 0;
    }

    GetRhsVar(4,"i",&m4,&n4,&l4); /* nbit */
    CheckDims(4,m4,n4,1,1);
    nbit_ptr = istk(l4);
    /* verification valeur nbit */
    if (*nbit_ptr<0) {
      Scierror(888,"%s : Number of bits must be positive.\n",fname);
      return 0;
    }

    l5 = I_INT32; /* y */
    CreateVar(5,"I",istk(l1),istk(l2),&l5);

    /*Appel genint_c*/
    geninti_c(istk(l1),istk(l2),nbit_ptr,typ_ptr,istk(l5));

    /* Retourne la variable de sortie */
    LhsVar(1) = 5;

    return 0;
  }

}

/*
 * y=genrayl(sig,n[,m])
 *
 * Gnrateur de nombres alatoires de Rayleigh
 *
 * Entres : sig  : paramtre (scalaire)
 *           n,m  : dimensions de la matrice de sortie
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgenrayl_c(char *fname)
{
  static int l1, m1, n1; /*entre 1*/
  static int l2, m2, n2; /*entre 2*/
  static int l3, m3, n3; /*entre 3*/
  int l4;                /*sortie 1*/

  int m=1;

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(2,3);
  CheckLhs(1,1);

  /* Rcupre entre 1 (sig) */
  GetRhsVar(1,"d",&m1,&n1,&l1); /* sig */
  CheckDims(1,m1,n1,1,1);

  /* Rcupre entre 2 (n) */
  GetRhsVar(2,"i",&m2,&n2,&l2); /* n */
  CheckDims(2,m2,n2,1,1);
  /* verification valeur n */
  if (*istk(l2)<=0) {
    Scierror(888,"%s : Number of row must be positive.\n",fname);
    return 0;
  }

  /* Rcupration entre 3 m et cre la sortie */
  if (Rhs==3) {
    GetRhsVar(3,"i",&m3,&n3,&l3); /* m */
    CheckDims(3,m3,n3,1,1);
    /* verification valeur m */
    if (*istk(l3)<=0) {
      Scierror(888,"%s : Number of col must be positive.\n",fname);
      return 0;
    }

    CreateVar(4,"d",istk(l2),&m,&l4); /* y */
    LhsVar(1) = 4;
  }
  else {
    CreateVar(3,"d",istk(l2),&m,&l4); /* y */
    LhsVar(1) = 3;
  }

  /*Appel rnoise_c*/
  rnoise_c(istk(l2),&m,stk(l1),stk(l4));

  return 0;

}

/*
 * y=upsmpl(u,nech[,typ,init])
 *
 * Sur chantilloneur (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  sur chantillonner
 *                  (int32 ou double) de dimensions n,m
 *           nech : facteur de sur-chantillonnage
 *           typ  : type de sur-chantillonnage (dfaut 0)
 *                  0 : sur-chantillonnage par rptition des valeurs
 *                  1 : sur-chantillonnage par insertion de zro
 *           init : chantillon inital o commence le sur-chantillonnage
 *                  (dfaut 0)
 *
 * Sorties : y : matrice de sortie de taille n*nech,m
 *
 */
int intsurecht_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3*/
  static int l4,m4,n4; /*entre 4*/
  int l5,m5;           /*sortie 1*/

  int *typ_ptr;              /* type de sortie */
  int typ=0;

  int *init_ptr;             /* compteur initial */
  int init=1;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(2,4);
  CheckLhs(1,1);

  /* Rcupre les entres et cre la sortie */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }
  else if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1);
  }

  GetRhsVar(2,"i",&m2,&n2,&l2); /*nech : facteur de surechantillonnage -scalaire-*/
  CheckDims(2,m2,n2,1,1);
  /* verification valeur nech */
  if (*istk(l2)<0) {
    Scierror(888,"%s : Upsample factor must be positive.\n",fname);
    return 0;
  }

  /* cas 2 paramtres */
  if (Rhs==2) {
    /* Rcupration typ/init */
    typ_ptr  = &typ;
    init_ptr = &init;

    /*Appel surech_c*/
    if (VarType(1)==1) {
      m5 = m1 * (*istk(l2));
      CreateVar(3,"d",&m5,&n1,&l5); /* y */

      surecht_c(typ_ptr,&m1,&n1,istk(l2),init_ptr,stk(l1),stk(l5));
    }
    else if (VarType(1)==8) {
      l5 = I_INT32; /* y */
      m5 = m1 * (*istk(l2));
      CreateVar(3,"I",&m5,&n1,&l5);

      surechti_c(typ_ptr,&m1,&n1,istk(l2),init_ptr,M1.D,istk(l5));
    }

    /*Retourne la variable de sortie*/
    LhsVar(1) = 3;

    return 0;
  }
  /* cas 3 paramtres */
  else if (Rhs==3) {
    /* Rcupration typ/init */
    GetRhsVar(3,"i",&m3,&n3,&l3); /* typ */
    CheckDims(3,m3,n3,1,1);
    typ_ptr = istk(l3);
    /* verification valeur typ */
    if ((*typ_ptr!=0)&&(*typ_ptr!=1)) {
      Scierror(888,"%s : Type of upsample must be 0 or 1.\n",fname);
      return 0;
    }

    init_ptr = &init;

    /*Appel surech_c*/
    if (VarType(1)==1) {
      m5 = m1 * (*istk(l2));
      CreateVar(4,"d",&m5,&n1,&l5); /* y */

      surecht_c(typ_ptr,&m1,&n1,istk(l2),init_ptr,stk(l1),stk(l5));
    }
    else if (VarType(1)==8) {
      l5 = I_INT32; /* y */
      m5 = m1 * (*istk(l2));
      CreateVar(4,"I",&m5,&n1,&l5);

      surechti_c(typ_ptr,&m1,&n1,istk(l2),init_ptr,M1.D,istk(l5));
    }

    /*Retourne la variable de sortie*/
    LhsVar(1) = 4;

    return 0;
  }
  /* cas 4 paramtres */
  else {
    /* Rcupration typ/init */
    GetRhsVar(3,"i",&m3,&n3,&l3); /* typ */
    CheckDims(3,m3,n3,1,1);
    typ_ptr = istk(l3);
    /* verification valeur typ */
    if ((*typ_ptr!=0)&&(*typ_ptr!=1)) {
      Scierror(888,"%s : Type of upsample must be 0 or 1.\n",fname);
      return 0;
    }

    GetRhsVar(4,"i",&m4,&n4,&l4); /* init */
    CheckDims(4,m4,n4,1,1);
    init_ptr = istk(l4);
    /* verification valeur init */
    if (*init_ptr<0) {
      Scierror(888,"%s : Initial counter value must be positive.\n",fname);
      return 0;
    }
    else if (*init_ptr>=*istk(l2)) {
      Scierror(888,"%s : Initial counter value must be less than upsample factor.\n",fname);
      return 0;
    }

    /*Appel surech_c*/
    if (VarType(1)==1) {
      m5 = m1 * (*istk(l2));
      CreateVar(5,"d",&m5,&n1,&l5); /* y */

      surecht_c(typ_ptr,&m1,&n1,istk(l2),init_ptr,stk(l1),stk(l5));
    }
    else if (VarType(1)==8) {
      l5 = I_INT32; /* y */
      m5 = m1 * (*istk(l2));
      CreateVar(5,"I",&m5,&n1,&l5);

      surechti_c(typ_ptr,&m1,&n1,istk(l2),init_ptr,M1.D,istk(l5));
    }

    /*Retourne la variable de sortie*/
    LhsVar(1) = 5;

    return 0;
  }

}

/*
 * y=downsmpl(u,nech[,init])
 *
 * Sous chantilloneur (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  sous chantillonner
 *                  (int32 ou double) de dimensions n,m
 *           nech : facteur de sur-chantillonnage
 *           init : chantillon inital o commence le sur-chantillonnage
 *                  (dfaut 0)
 *
 * Sorties : y : matrice de sortie de taille int(n/nech),m
 *
 */
int intsousecht_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3*/
  int l4,m4;           /*sortie 1*/

  int *init_ptr;             /* compteur initial */
  int init=1;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(2,3);
  CheckLhs(1,1);

  /* Rcupre les entres et cre la sortie */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }
  else if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1);
  }

  GetRhsVar(2,"i",&m2,&n2,&l2); /*nech : facteur de surechantillonnage -scalaire-*/
  CheckDims(2,m2,n2,1,1);
  /* verification valeur nech */
  if (*istk(l2)<0) {
    Scierror(888,"%s : Upsample factor must be positive.\n",fname);
    return 0;
  }

  /* cas 2 paramtres */
  if (Rhs==2) {
    /* Rcupration typ/init */
    init_ptr = &init;

    /*Appel sousecht_c*/
    if (VarType(1)==1) {
      m4 = (int) (m1 / (*istk(l2)));
      CreateVar(3,"d",&m4,&n1,&l4); /* y */

      sousecht_c(&m1,&n1,istk(l2),init_ptr,stk(l1),stk(l4));
    }
    else if (VarType(1)==8) {
      l4 = I_INT32; /* y */
      m4 = (int) (m1 / (*istk(l2)));
      CreateVar(3,"I",&m4,&n1,&l4);

      sousechti_c(&m1,&n1,istk(l2),init_ptr,M1.D,istk(l4));
    }

    /*Retourne la variable de sortie*/
    LhsVar(1) = 3;

    return 0;
  }
  /* cas 3 paramtres */
  else {
    GetRhsVar(3,"i",&m3,&n3,&l3); /* init */
    CheckDims(3,m3,n3,1,1);
    init_ptr = istk(l3);
    /* verification valeur init */
    if (*init_ptr<0) {
      Scierror(888,"%s : Initial counter value must be positive.\n",fname);
      return 0;
    }
    else if (*init_ptr>=*istk(l2)) {
      Scierror(888,"%s : Initial counter value must be less than downsample factor.\n",fname);
      return 0;
    }

    /*Appel sousech_c*/
    if (VarType(1)==1) {
      m4 = (int) (m1 / (*istk(l2)));
      CreateVar(4,"d",&m4,&n1,&l4); /* y */

      sousecht_c(&m1,&n1,istk(l2),init_ptr,stk(l1),stk(l4));
    }
    else if (VarType(1)==8) {
      l4 = I_INT32; /* y */
      m4 = (int) (m1 / (*istk(l2)));
      CreateVar(4,"I",&m4,&n1,&l4);

      sousechti_c(&m1,&n1,istk(l2),init_ptr,M1.D,istk(l4));
    }

    /*Retourne la variable de sortie*/
    LhsVar(1) = 4;

    return 0;
  }

}

/*
 * [yi,yq]=modpsk(u,ns)
 *
 * Modulateur psk (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  moduler par
 *                  tat de phase (int32 ou double) de dimensions n,m
 *           ns   : nombre d'tat de phase
 *
 * Sorties : yi : matrice de sortie de taille n,m composante i (double)
 *           yq : matrice de sortie de taille n,m composante q (double)
 *
 *
 */
int intmodpsk_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3; /*sortie 2*/
  static int l4; /*sortie 1*/

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(2,2);
  CheckLhs(2,2);

  /* Rcupre les entres et cre la sortie */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }
  else if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1); /* u */
  }

  GetRhsVar(2,"i",&m2,&n2,&l2); /*ns : nombre d'tats de la constellation*/
  CheckDims(2,m2,n2,1,1);
  /* verification valeur ns */
  if (*istk(l2)<0) {
    Scierror(888,"%s : Number of states must be positive.\n",fname);
    return 0;
  }

  CreateVar(3,"d",&m1,&n1,&l3); /* yi */
  CreateVar(4,"d",&m1,&n1,&l4); /* yq */

  /*Appel modpsk_c*/
  if (VarType(1)==1) {
    modpsk_c(&m1,&n1,istk(l2),stk(l1),stk(l3),stk(l4));
  }
  else if (VarType(1)==8) {
    modpski_c(&m1,&n1,istk(l2),M1.D,stk(l3),stk(l4));
  }

  /*Retourne les variables de sortie*/
  LhsVar(1) = 3;
  LhsVar(2) = 4;

  return 0;
}

/*
 * [y]=demodpsk(ui,uq,ns)
 *
 * Demodulateur psk (double)
 *
 * Entres : ui   : scalaire, vecteur, matrice  dmoduler par
 *                  tat de phase (double) de dimensions n,m
 *                  composante i
 *           uq   : scalaire, vecteur, matrice  dmoduler par
 *                  tat de phase (double) de dimensions n,m
 *                  composante q
 *           ns   : nombre d'tat de phase
 *
 * Sorties : y : matrice de sortie de taille n,m (double)
 *
 *
 */

int intdemodpsk_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3*/
  static int l4; /*sortie 1*/

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(3,3);
  CheckLhs(1,1);

  /* Rcupre les entres et cre la sortie */
  GetRhsVar(1,"d",&m1,&n1,&l1); /* ui */

  GetRhsVar(2,"d",&m2,&n2,&l2); /* uq */
  /* vrification tailles ui,uq */
  CheckDims(2,m2,n2,m1,n1);

  GetRhsVar(3,"i",&m3,&n3,&l3); /*ns : nombre d'tats de la constellation*/
  CheckDims(3,m3,n3,1,1);

  /* verification valeur ns */
  if (*istk(l3)<0) {
    Scierror(888,"%s : Number of states must be positive.\n",fname);
    return 0;
  }

  CreateVar(4,"d",&m1,&n1,&l4); /* y */

  /*Appel demodpsk_c*/
  demodpsk_c(&m1,&n1,istk(l3),stk(l1),stk(l2),stk(l4));

  /*Retourne les variables de sortie*/
  LhsVar(1) = 4;

  return 0;
}

/*
 * [yi,yq]=modqam(u,ns)
 *
 * Modulateur qam (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  moduler en
 *                  phase/amplitude (int32 ou double) de dimensions n,m
 *           ns   : nombre d'tat (doit tre un nombre gal  2^nbit avec
                   nbit pair)
 *
 * Sorties : yi : matrice de sortie de taille n,m composante i (int32 ou double)
 *           yq : matrice de sortie de taille n,m composante q (int32 ou double)
 *
 *
 */
int intmodqam_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3; /*sortie 1*/
  static int l4; /*sortie 2*/

  int nb_i;
  double nb_d;

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(2,2);
  CheckLhs(2,2);

  /* Rcupre les entres et cre la sortie */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }
  else {
    GetRhsVar(1,"I",&m1,&n1,&M1); /* u */
  }

  GetRhsVar(2,"i",&m2,&n2,&l2); /*ns : nombre d'tats de la constellation*/
  CheckDims(2,m2,n2,1,1);

  /* verification valeur ns */
  if (*istk(l2)<0) {
    Scierror(888,"%s : Number of states must be positive.\n",fname);
    return 0;
  }
  nb_d = log(*istk(l2))/log(2);
  nb_i = (int) (nb_d/2);
  nb_d = nb_d/2;
  if (nb_d!=(double)nb_i) {
    Scierror(888,"%s : Number of states must have a value equal to 2^n with n even.\n",fname);
    return 0;
  }
  nb_i = (int) (nb_d*2);

  /* Appel modqam_c */
  if (VarType(1)==1) {
    CreateVar(3,"d",&m1,&n1,&l3); /* yi */
    CreateVar(4,"d",&m1,&n1,&l4); /* yq */

    modqam_c(&m1,&n1,&nb_i,stk(l1),stk(l3),stk(l4));
  }
  else {
    l3 = I_INT32; /* yi */
    l4 = I_INT32; /* yq */
    CreateVar(3,"I",&m1,&n1,&l3); /* yi */
    CreateVar(4,"I",&m1,&n1,&l4); /* yq */

    modqami_c(&m1,&n1,&nb_i,M1.D,istk(l3),istk(l4));
  }

  /*Retourne les variables de sortie*/
  LhsVar(1) = 3;
  LhsVar(2) = 4;

  return 0;
}

/*
 * [y]=demodqam(ui,uq,ns)
 *
 * Demodulateur psk (double)
 *
 * Entres : ui   : scalaire, vecteur, matrice  dmoduler par
 *                  phase/amplitude (double ou int32) de dimensions n,m
 *                  composante i
 *           uq   : scalaire, vecteur, matrice  dmoduler par
 *                  phase/amplitude (double ou int32) de dimensions n,m
 *                  composante q
 *           ns   : nombre d'tat (doit tre un nombre gal  2^nbit avec
                   nbit pair)
 *
 * Sorties : y : matrice de sortie de taille n,m (double ou int32)
 *
 */

int intdemodqam_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  SciIntMat M2;
  static int l3,m3,n3; /*entre 3*/
  static int l4; /*sortie 1*/

  int nb_i;
  double nb_d;

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(3,3);
  CheckLhs(1,1);

  /* Rcupre les entres et cre la sortie */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* ui */
    GetRhsVar(2,"d",&m2,&n2,&l2); /* uq */
  }
  else {
    GetRhsVar(1,"I",&m1,&n1,&M1); /* ui */
    GetRhsVar(2,"I",&m2,&n2,&M2); /* uq */
  }

  /* vrification tailles ui,uq */
  CheckDims(2,m2,n2,m1,n1);

  GetRhsVar(3,"i",&m3,&n3,&l3); /*ns : nombre d'tats de la constellation*/
  CheckDims(3,m3,n3,1,1);

  /* verification valeur ns */
  if (*istk(l3)<0) {
    Scierror(888,"%s : Number of states must be positive.\n",fname);
    return 0;
  }
  nb_d = log(*istk(l3))/log(2);
  nb_i = (int) (nb_d/2);
  nb_d = nb_d/2;
  if (nb_d!=(double)nb_i) {
    Scierror(888,"%s : Number of states must have a value equal to 2^n with n even.\n",fname);
    return 0;
  }
  nb_i = (int) (nb_d*2);


  /*Appel demodqam_c*/
  if (VarType(1)==1) {
    CreateVar(4,"d",&m1,&n1,&l4); /* y */

    demodqam_c(&m1,&n1,&nb_i,stk(l1),stk(l2),stk(l4));
  }
  else {
    l4 = I_INT32; /* y */
    CreateVar(4,"I",&m1,&n1,&l4); /* y */

    demodqami_c(&m1,&n1,&nb_i,M1.D,M2.D,istk(l4));
  }

  /*Retourne les variables de sortie*/
  LhsVar(1) = 4;

  return 0;
}

/*
 * y=gengold(lreg,ci,coef[,n])
 *
 * gnrateur de squences de gold (int32)
 *
 * Entres : lreg : longueur des registres
 *           ci : conditions initiales
 *                ci = [ci1 ; ci2]
 *           coef : coefficients
 *                  coef = [coef1 ; coef2]
                 les tailles de ci1/ci2 et
 *               de coef1/coef2 doivent tre 1,m
 *           n : tailles des sorties
 *               dfaut : n = 2^(lreg-1)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgengold_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3*/
  static int l4,m4,n4; /*entre 4*/
  int ls;              /*sortie 1*/

  int ngen; /* nombre de gnrateurs */
  int ly;   /* longueur de la sortie */

  int *coef1;
  int *coef2;
  int *reg1;
  int *reg2;
  int *ys;

  int i;

  ls = I_INT32; /* y */

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(3,4);
  CheckLhs(1,1);

  /* verification lreg */
  GetRhsVar(1,"i",&m1,&n1,&l1); /* lreg */
  CheckDims(1,m1,n1,1,1);

  /* verification ci */
  GetRhsVar(2,"i",&m2,&n2,&l2); /* ci */
  if (m2!=2) {
    Scierror(888,"%s : Initial conditions must have a size (2,ngen).\n",fname);
    return 0;
  }
  ngen = n2;

  /* verification coef */
  GetRhsVar(3,"i",&m3,&n3,&l3); /* coef */
  CheckDims(3,m3,n3,2,ngen);

  /* cas 4 paramtres */
  if (Rhs==4) {
    GetRhsVar(4,"i",&m4,&n4,&l4); /* n */
    CheckDims(4,m4,n4,1,1);
    ly = *istk(l4);
    CreateVar(5,"I",&ly,&ngen,&ls); /* y */
  }
  else {
    ly = (int) pow(2,(*istk(l1)))-1;
    CreateVar(4,"I",&ly,&ngen,&ls); /* y */
  }

  /* appel goldi_c */
  for(i=0;i<ngen;i++) {
    reg1  = istk(l2+(2*i));
    reg2  = istk(l2+1+(2*i));
    coef1 = istk(l3+(2*i));
    coef2 = istk(l3+1+(2*i));
    ys    = istk(ls+i*ly);

    goldi_c(istk(l1), &ly, ys, reg1, reg2, coef1, coef2);
  }

  /* retourne la sortie */
  if (Rhs==4) {
    LhsVar(1) = 5;
  }
  else {
    LhsVar(1) = 4;
  }

  return 0;
}

/*
 * y=genpn(lreg,ci,coef[,n])
 *
 * gnrateur de squences PN (int32)
 *
 * Entres : lreg : longueur des registres
 *           ci : conditions initiales
 *           coef : coefficients
                 les tailles de ci et
 *               de coef doivent tre 1,m
 *           n : tailles des sorties
 *               dfaut : n = 2^(lreg-1)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgenpn_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3*/
  static int l4,m4,n4; /*entre 4*/
  int ls;              /*sortie 1*/

  int ngen; /* nombre de gnrateurs */
  int ly;   /* longueur de la sortie */

  int *coef;
  int *reg;
  int *ys;

  int i;

  ls = I_INT32; /* y */

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(3,4);
  CheckLhs(1,1);

  /* verification lreg */
  GetRhsVar(1,"i",&m1,&n1,&l1); /* lreg */
  CheckDims(1,m1,n1,1,1);

  /* verification ci */
  GetRhsVar(2,"i",&m2,&n2,&l2); /* ci */
  if (m2!=1) {
    Scierror(888,"%s : Initial conditions must have a size (1,ngen).\n",fname);
    return 0;
  }
  ngen = n2;

  /* verification coef */
  GetRhsVar(3,"i",&m3,&n3,&l3); /* coef */
  CheckDims(3,m3,n3,1,ngen);

  /* cas 4 paramtres */
  if (Rhs==4) {
    GetRhsVar(4,"i",&m4,&n4,&l4); /* n */
    CheckDims(4,m4,n4,1,1);
    ly = *istk(l4);
    CreateVar(5,"I",&ly,&ngen,&ls); /* y */
  }
  else {
    ly = (int) pow(2,(*istk(l1)))-1;
    CreateVar(4,"I",&ly,&ngen,&ls); /* y */
  }

  /* appel mllsrsi_c */
  for(i=0;i<ngen;i++) {
    reg   = istk(l2+i);
    coef  = istk(l3+i);
    ys    = istk(ls+i*ly);

    mllsrsi_c(istk(l1), &ly, ys, reg, coef);
  }

  /* retourne la sortie */
  if (Rhs==4) {
    LhsVar(1) = 5;
  }
  else {
    LhsVar(1) = 4;
  }

  return 0;
}

/*
 * y=spread(u,c)
 *
 * insertion de code
 *
 * Entres : u : vecteur/matrice symbole  taler (de taille n,m)
 *           c : vecteur/matrice code (de taille nc,m)
 *
 * Sorties : y : vecteur/matrice de sortie (de taille n*nc,m)
 *
 */
int intspread_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  SciIntMat M2;
  static int l3,m3,n3; /*entre 3*/

  int typf; /* type de la fonction  appeler */

  double *c_ptr;
  int *c_ptri;
  double *u_ptr;
  int *u_ptri;
  double *y_ptr;
  int *y_ptri;

  int i;

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(2,2);
  CheckLhs(1,1);

  /* u */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1);
    typf = 0;
  }
  else if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1);
    typf = 1;
  }

  /* c */
  if (VarType(2)==1) {
    GetRhsVar(2,"d",&m2,&n2,&l2);
  }
  else if (VarType(2)==8) {
    GetRhsVar(2,"I",&m2,&n2,&M2);
    typf = typf + 2;
  }

  /* verification tailles */
  if (n1!=n2) {
    Scierror(888,"%s : symbols input and chips input must have the same numbers of cols.\n",fname);
    return 0;
  }

  /* appel codinser_c */
  switch(typf)
  {
   case 0 : /* u et c double : y double */
            CreateVar(3,"d",(i=(m1*m2),&i),&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptr = stk(l2+i*m2);
              u_ptr = stk(l1+i*m1);
              y_ptr = stk(l3+i*(m1*m2));
              codinser_c(&m2,&m1,c_ptr,u_ptr,y_ptr);
            }
            break;

   case 1 : /* u int et c double : y int */
            l3 = I_INT32;
            CreateVar(3,"I",(i=(m1*m2),&i),&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptr  = stk(l2+i*m2);
              u_ptri = (int *) ((int *)M1.D + i*m1);
              y_ptri = istk(l3+i*(m1*m2));
              codinserid_c(&m2,&m1,c_ptr,u_ptri,y_ptri);
            }
            break;

   case 2 : /* u double et c int : y double */
            CreateVar(3,"d",(i=(m1*m2),&i),&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptri = (int *) ((int *)M2.D + i*m2);
              u_ptr  = stk(l1+i*m1);
              y_ptr  = stk(l3+i*(m1*m2));
              codinserdi_c(&m2,&m1,c_ptri,u_ptr,y_ptr);
            }
            break;

   case 3 : /* u et c int : y int */
            l3 = I_INT32;
            CreateVar(3,"I",(i=(m1*m2),&i),&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptri = (int *) ((int *)M2.D + i*m2);
              u_ptri = (int *) ((int *)M1.D + i*m1);
              y_ptri = istk(l3+i*(m1*m2));
              codinseri_c(&m2,&m1,c_ptri,u_ptri,y_ptri);
            }
            break;
  }

  LhsVar(1) = 3;
  return 0;
}

/*
 * y=despread(u,c)
 *
 * deinsertion de code
 *
 * Entres : u : vecteur/matrice symbole  de-taler (de taille n*nc,m)
 *           c : vecteur/matrice code (de taille nc,m)
 *
 * Sorties : y : vecteur/matrice de sortie (de taille n*nc,m)
 *
 */
int intdespread_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  SciIntMat M2;
  static int l3,m3,n3; /*entre 3*/

  int typf; /* type de la fonction  appeler */

  double *c_ptr;
  int *c_ptri;
  double *u_ptr;
  int *u_ptri;
  double *y_ptr;
  int *y_ptri;

  int i;

  /* Vrification du nombre d'entres/sorties */
  CheckRhs(2,2);
  CheckLhs(1,1);

  /* u */
  if (VarType(1)==1) {
    GetRhsVar(1,"d",&m1,&n1,&l1);
    typf = 0;
  }
  else if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1);
    typf = 1;
  }

  /* c */
  if (VarType(2)==1) {
    GetRhsVar(2,"d",&m2,&n2,&l2);
  }
  else if (VarType(2)==8) {
    GetRhsVar(2,"I",&m2,&n2,&M2);
    typf = typf + 2;
  }

  /* verification tailles */
  if ((n1/n2)!=(double)((int)(n1/n2))) {
    Scierror(888,"%s : size of symbols input and chips input doesn't match. Please check.\n",fname);
    return 0;
  }

  /* appel decod_c */
  switch(typf)
  {
   case 0 : /* u et c double : y double */
            CreateVar(3,"d",&m1,&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptr = stk(l2+i*m2);
              u_ptr = stk(l1+i*m1);
              y_ptr = stk(l3+i*m1);
              decod_c(&m2,&m1,c_ptr,u_ptr,y_ptr);
            }
            break;

   case 1 : /* u int et c double : y int */
            l3 = I_INT32;
            CreateVar(3,"I",&m1,&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptr  = stk(l2+i*m2);
              u_ptri = (int *) ((int *)M1.D + i*m1);
              y_ptri = istk(l3+i*m1);
              decodid_c(&m2,&m1,c_ptr,u_ptri,y_ptri);
            }
            break;

   case 2 : /* u double et c int : y double */
            CreateVar(3,"d",&m1,&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptri = (int *) ((int *)M2.D + i*m2);
              u_ptr  = stk(l1+i*m1);
              y_ptr  = stk(l3+i*m1);
              decoddi_c(&m2,&m1,c_ptri,u_ptr,y_ptr);
            }
            break;

   case 3 : /* u et c int : y int */
            l3 = I_INT32;
            CreateVar(3,"I",&m1,&n1,&l3); /* y */

            for (i=0;i<n1;i++) {
              c_ptri = (int *) ((int *)M2.D + i*m2);
              u_ptri = (int *) ((int *)M1.D + i*m1);
              y_ptri = istk(l3+i*m1);
              decodi_c(&m2,&m1,c_ptri,u_ptri,y_ptri);
            }
            break;
  }

  LhsVar(1) = 3;
  return 0;
}

/*
 * [y[,z]]=nfilter(u,h[,z])
 *
 * Banc de filtres  rponse impulsionnelle fini.
 *
 * Entres : u  : vecteur/matrice  filtrer (de taille n,m)
 *           h  : vecteur de la rponse impulsionnelle (de taille nbcoef)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 * E/S       z : tats (de taille nbcoef,m) du filtre
 */
int intnfilter_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*sortie 2*/
  static int l4; /*sortie 1*/
  int i;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(2,3);
  CheckLhs(1,2);

  /* Rcupre les entres et cre les sorties */
  if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1); /* u */
  }
  else {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }

  GetRhsVar(2,"d",&m2,&n2,&l2); /* h */

  if (Rhs==3) {
    /* Rcupration z */
    GetRhsVar(3,"d",&m3,&n3,&l3); /* z */
    /* verification valeur nbcoef/n1 */
    CheckDims(3,m3,n3,m2,n1);
  }
  else {
    /* Rcupration z */
    CreateVar(3,"d",&m2,&n1,&l3); /* z */
    for(i=0;i<m2*n1;i++) {
      *(stk(l3) + i) = 0.;
    }
  }

  CreateVar(4,"d",&m1,&n1,&l4); /* y */

  /*Appel nfilter_c*/
  if (VarType(1)==1) {
    nfilter_c(&m1,&n1,&m2,stk(l1),stk(l2),stk(l4),stk(l3));
  }
  else {
    nfilteri_c(&m1,&n1,&m2,M1.D,stk(l2),stk(l4),stk(l3));
  }

  /*Retourne les variables de sortie*/
  LhsVar(1) = 4;
  if (Lhs==2) {
    LhsVar(2) = 3;
  }

  return 0;
}

/*
 * y = awgn(u,sig[,mean])
 *
 * [yi,yq] = awgn(ui,uq,sig[,mean])
 *
 * Canal  bruit blanc gaussien additif
 *
 * Entres : u,[ui,uq] : vecteurs/matrices  bruiter (de taille n,m) (double ou int32)
 *           sig   : variance de la source de bruit blanc (scalaire)
 *           mean  : moyenne de la source de bruit blanc (ou composante DC) (scalaire)
 *
 * Sorties : y,[yi,yq] : matrice(s) de sortie de taille n,m (double)
 *
 */
int intawgn_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  SciIntMat M2;
  static int l3,m3,n3; /*entre 3*/
  static int l4,m4,n4; /*entre 4*/
  static int l5; /*sortie 1*/
  static int l6; /*sortie 2*/

  double mean;

  int i;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(2,4);
  CheckLhs(1,2);

  /* cas i/q */
  if (Lhs==2) {
    /*vrification nombre de paramtres*/
    if (Rhs==2) {
      Scierror(888,"%s : With two ouputs, the number of inputs must at less 3.\n",fname);
      return 0;
    }

    /* Rcupre les entres ui,uq */
    if (VarType(1)==8) {
      GetRhsVar(1,"I",&m1,&n1,&M1); /* ui */
      GetRhsVar(2,"I",&m2,&n2,&M2); /* uq */
    }
    else {
      GetRhsVar(1,"d",&m1,&n1,&l1); /* ui */
      GetRhsVar(2,"d",&m2,&n2,&l2); /* uq */
    }

    /* vrification tailles ui,uq */
    CheckDims(2,m2,n2,m1,n1);

    /* Rcupre l'entre sig */
    GetRhsVar(3,"d",&m3,&n3,&l3); /*sig : variance de la source de bruit*/
    CheckDims(3,m3,n3,1,1);

    /* verification valeur sig */
    if (*stk(l3)<0) {
      Scierror(888,"%s : Variance must be positive.\n",fname);
      return 0;
    }

    /* Rcupre l'entre mean et cre les sorties*/
    if (Rhs==4) {
      GetRhsVar(4,"d",&m4,&n4,&l4); /*mean : composante DC*/
      CheckDims(4,m4,n4,1,1);

      mean = *stk(l4);

      CreateVar(5,"d",&m1,&n1,&l5); /* yi */
      CreateVar(6,"d",&m1,&n1,&l6); /* yq */

      LhsVar(1) = 5;
      LhsVar(2) = 6;
    }
    else {
      mean = 0.;

      CreateVar(4,"d",&m1,&n1,&l5); /* yi */
      CreateVar(5,"d",&m1,&n1,&l6); /* yq */

      LhsVar(1) = 4;
      LhsVar(2) = 5;
    }

    /* appel awgn_c */
    if (VarType(1)==1) {
      awgniq_c(&m1,&n1,stk(l3),&mean,stk(l1),stk(l2),stk(l5),stk(l6));
    }
    else {
      awgniqi_c(&m1,&n1,stk(l3),&mean,M1.D,M2.D,stk(l5),stk(l6));
    }

  }
  /* cas une seule composante */
  else {
    /*vrification nombre de paramtres*/
    if (Rhs==4) {
      Scierror(888,"%s : With one ouput, the number of ths parameters must be max 3.\n",fname);
      return 0;
    }

    /* Rcupre l'entre u */
    if (VarType(1)==8) {
      GetRhsVar(1,"I",&m1,&n1,&M1); /* u */
    }
    else {
      GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
    }

    /* Rcupre l'entre sig */
    GetRhsVar(2,"d",&m2,&n2,&l2); /*sig : variance de la source de bruit*/
    CheckDims(2,m2,n2,1,1);

    /* verification valeur sig */
    if (*stk(l2)<0) {
      Scierror(888,"%s : Variance must be positive.\n",fname);
      return 0;
    }

    /* Rcupre l'entre mean et cre les sorties*/
    if (Rhs==3) {
      GetRhsVar(3,"d",&m3,&n3,&l3); /*mean : composante DC*/
      CheckDims(3,m3,n3,1,1);

      mean = *stk(l3);

      CreateVar(4,"d",&m1,&n1,&l4); /* y */

      LhsVar(1) = 4;
    }
    else {
      mean = 0.;

      CreateVar(3,"d",&m1,&n1,&l4); /* y */

      LhsVar(1) = 3;
    }

    /* appel awgn_c */
    if (VarType(1)==1) {
      awgn_c(&m1,&n1,stk(l2),&mean,stk(l1),stk(l4));
    }
    else {
      awgni_c(&m1,&n1,stk(l2),&mean,M1.D,stk(l4));
    }

  }

  return 0;
}

/*
 * [y[,z[,initc]]] = intsym(u,nech[,initc,step,z])
 *
 * Intgrateur discret avec remise  zro
 *
 * Entres : u     : vecteurs/matrices  intgrer (de taille n,m) (double ou int32)
 *           nech  : nombre d'chantillons par symbole (scalaire)
 *           initc : chantillons initiaux o l'intgration commence
 *                   compris entre 1 et nech (vecteur de taille m)
 *                   (defaut 1)
 *           step  : pas d'intgration (scalaire) (defaut 1)
 *
 * Sorties : y : matrice(s) de sortie de taille n,m (double)
 *
 * Entre/Sortie : z : tat du filtre (matrice de taille m,2 - double)
 *
 */
int intintsym_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3*/
  static int l4,m4,n4; /*entre 4*/
  static int l5,m5,n5; /*entre 5*/
  static int l6; /*sortie 1*/
  static int l7; /*sortie 2*/
  static int l8; /*sortie 3*/

  int i,j;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(2,5);
  CheckLhs(1,3);

  /* Rcupre l'entre u */
  if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1); /* u */
  }
  else {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }

  /* Rcupre l'entre nech */
  GetRhsVar(2,"i",&m2,&n2,&l2); /* nech */
  CheckDims(2,m2,n2,1,1);

  /* Rcupre l'entre initc */
  if(Rhs>=3) {
    GetRhsVar(3,"i",&m3,&n3,&l3); /* initc */
    if (m3*n3!=n1) {
      Scierror(888,"%s : Bad dimensions for third input.\n",fname);
      return 0;
    }
  }
  else {
    CreateVar(3,"i",&n1,(i=1,&i),&l3); /* initc */
    for (i=0;i<n1;i++) {
      *istk(l3+i) = 0;
    }
  }

  /* Rcupre l'entre step */
  if(Rhs>=4) {
    GetRhsVar(4,"d",&m4,&n4,&l4); /* step */
    CheckDims(4,m4,n4,1,1);
  }
  else {
    CreateVar(4,"d",(i=1,&i),(j=1,&j),&l4); /* step */
    *stk(l4) = 1.;
  }

  /* Rcupre l'entre z */
  if(Rhs>=5) {
    GetRhsVar(5,"d",&m5,&n5,&l5); /* z */
    if (m5*n5!=n1*2) {
      Scierror(888,"%s : Bad dimensions for states of filter.\n",fname);
      return 0;
    }
  }
  else {
    CreateVar(5,"d",&n1,(i=2,&i),&l5); /* z */
    for (i=0;i<n1*2;i++) {
      *stk(l5+i) = 0.;
    }
  }

  /* cre la sortie */
  CreateVar(6,"d",&m1,&n1,&l6); /* y */

  if (Lhs==3) {
    CreateVar(7,"d",&n1,(i=1,&i),&l7); /* init_c */
  }

  /* appel intsym_c */
  if (VarType(1)==1) {
    intsym_c(&m1,&n1,istk(l2),istk(l3),stk(l4),stk(l1),stk(l6),stk(l5));
  }
  else {
    intsymi_c(&m1,&n1,istk(l2),istk(l3),stk(l4),M1.D,stk(l6),stk(l5));
  }

  LhsVar(1)=6;
  if (Lhs>=2) {
    LhsVar(2)=5;
    if (Lhs==3) {
      for(i=0;i<n1;i++) {
        *((double *) stk(l7) + i)= (double) *((int *) istk(l3) + i);
      }
      LhsVar(3)=7;
    }
  }
  return 0;
}

/*
 * [y[,mean]]=autocorr(u[,lag,mean])
 *
 * Calcul d'une autocorrlation discrte
 *
 * Entres : u  : vecteur/matrice  filtrer (de taille n,m)
 *         lag  : longueur de l'autocorr dsire (scalaire)
 *                (default n)
 *         mean : moyenne optionnelle  soustraire pendant le
 *                calcul de l'autocorr (dfaut 0) (vecteur taille m)
 *
 * Sorties : y : matrice de sortie de taille lag,m
 *
 */
int intautocorr_c(char *fname)
{
  static int l1,m1,n1; /*entre 1*/
  SciIntMat M1;
  static int l2,m2,n2; /*entre 2*/
  static int l3,m3,n3; /*entre 3 (mean_u1)*/
  static int l4;       /*entre 4 (mean_u2)*/
  static int l5;       /*sortie 1*/

  int lag;

  int i;

  /* Vrification du nombre d'entres/sorties (lhs=1) */
  CheckRhs(1,3);
  CheckLhs(1,1);

  /* Rcupre les entres et cre les sorties */
  if (VarType(1)==8) {
    GetRhsVar(1,"I",&m1,&n1,&M1); /* u */
  }
  else {
    GetRhsVar(1,"d",&m1,&n1,&l1); /* u */
  }

  /* rcupration de lag */
  if (Rhs>=2) {
    GetRhsVar(2,"i",&m2,&n2,&l2); /* lag */
    CheckDims(2,m2,n2,1,1);
    lag = *istk(l2);
    /*vrification valeur lag*/
    if ((lag<=0) || (lag>(m1))) {
      Scierror(888,"%s : length of autocorrelation must be positive and\n"
                   "less or equal than the first dimension of u.\n",fname);
      return 0;
    }
  }
  else {
    lag = m1;
  }

  /* rcupration de mean et cre la sortie */
  if (Rhs==3) {
    GetRhsVar(3,"d",&m3,&n3,&l3); /* mean_u1 */
    CheckDims(3,m3,n3,1,n1);
    CreateVar(4,"d",(i=1,&i),&n1,&l4); /* mean_u2 */
    CreateVar(5,"d",&lag,&n1,&l5);     /* sortie */
    LhsVar(1)=5;
    if (Lhs==2) LhsVar(2)=3;
  }
  else {
    if (Rhs==1) {
      CreateVar(2,"d",(i=1,&i),&n1,&l3); /* mean_u1 */
      CreateVar(3,"d",(i=1,&i),&n1,&l4); /* mean_u2 */
      CreateVar(4,"d",&lag,&n1,&l5);     /* sortie */
      LhsVar(1)=4;
      if (Lhs==2) LhsVar(2)=2;
    }
    else {
      CreateVar(3,"d",(i=1,&i),&n1,&l3); /* mean_u1 */
      CreateVar(4,"d",(i=1,&i),&n1,&l4); /* mean_u2 */
      CreateVar(5,"d",&lag,&n1,&l5);     /* sortie */
      LhsVar(1)=5;
      if (Lhs==2) LhsVar(2)=3;
    }
  }

  /*Appel autocorr_c*/
  if (VarType(1)==1) {
    if (Rhs==3) {
      corr_c(&m1,&n1,(i=0,&i),&lag,stk(l3),stk(l4),stk(l1),stk(l1),stk(l5));
    }
    else {
      corr_c(&m1,&n1,(i=1,&i),&lag,stk(l3),stk(l4),stk(l1),stk(l1),stk(l5));
    }
  }
  else {
    if (Rhs==3) {
      corri_c(&m1,&n1,(i=0,&i),&lag,stk(l3),stk(l4),M1.D,M1.D,stk(l5));
    }
    else {
      corri_c(&m1,&n1,(i=1,&i),&lag,stk(l3),stk(l4),M1.D,M1.D,stk(l5));
    }
  }


  return 0;
}

static GenericTable Tab[]={
{(Myinterfun)sci_gateway, intautocorr_c,"autocorr"},
{(Myinterfun)sci_gateway, intawgn_c,"awgn"},
{(Myinterfun)sci_gateway, intdemodpsk_c,"demodpsk"},
{(Myinterfun)sci_gateway, intdemodqam_c,"demodqam"},
{(Myinterfun)sci_gateway, intdespread_c,"despread"},
{(Myinterfun)sci_gateway, intsousecht_c,"downsmpl"},
{(Myinterfun)sci_gateway, intgengold_c,"gengold"},
{(Myinterfun)sci_gateway, intgenint_c,"genint"},
{(Myinterfun)sci_gateway, intgenpn_c,"genpn"},
{(Myinterfun)sci_gateway, intgenrayl_c,"genrayl"},
{(Myinterfun)sci_gateway, intintsym_c,"intsym"},
{(Myinterfun)sci_gateway, intmodpsk_c,"modpsk"},
{(Myinterfun)sci_gateway, intmodqam_c,"modqam"},
{(Myinterfun)sci_gateway, intnfilter_c,"nfilter"},
{(Myinterfun)sci_gateway, intspread_c,"spread"},
{(Myinterfun)sci_gateway, intsurecht_c,"upsmpl"},
};

int C2F(intmodnum_lib)()
{  Rhs = Max(0, Rhs);
(*(Tab[Fin-1].f))(Tab[Fin-1].name,Tab[Fin-1].F);
  return 0;
}
