/* awgn_c subroutine
 * additive white gaussian noise channel
 * with Box Muller Law method
 * for complex values
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <math.h>
#include <stdlib.h>  /*pour RAND_MAX*/

#include "modnum_lib.h"

/* awgn_c routine de calcul d'un canal  bruit blanc gaussien additif
 * (chantillons bruits par la mthode "Box Muller Law")
 *
 * Entres :
 *   n   : dimension 1 des matrices d'entre/sortie (scalaire)
 *   m   : dimension 2 des matrices d'entre/sortie (scalaire)
 *  sig  : variance (scalaire)
 *  mean : moyenne (scalaire)
 *   u   : matrice en entre (double)
 * Sorties :
 *   y   : matrice en sortie (double)
 *
 * dpendances
 * math.h
 */

void awgn_c(int *n,int *m,double *sig,double *mean,double *u,double *y)
{
 /*dclaration des variables*/
 int i,l;
 double rand1, rand2;
 double rand_m;
 double ampl, phase;

 /*rcupration de la valeur de RAND_MAX*/
 rand_m=RAND_MAX;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*calcul rand1*/
     rand1=rand()/rand_m;
     /*test rand1*/
     while((rand1<=0)||(rand1>=1)) rand1=rand()/rand_m;

     /*calcul rand2*/
     rand2=rand()/rand_m;
     /*test rand2*/
     while((rand2<=0)||(rand2>=1)) rand2=rand()/rand_m;

     /*Calcul amplitude et phase*/
     ampl=(*sig)*sqrt(2*-log(rand1));
     phase=2*M_PI*rand2;

     /*Calcul y*/
     y[(*n)*l+i]=u[(*n)*l+i]+(*mean)+ampl*cos(phase);
   }
 }

 return;
}

void awgniq_c(int *n,int *m,double *sig,double *mean,\
              double *u_i,double *u_q,\
              double *y_i,double *y_q)
{
 /*dclaration des variables*/
 int i,l;
 double rand1, rand2;
 double rand_m;
 double ampl, phase;

 /*rcupration de la valeur de RAND_MAX*/
 rand_m=RAND_MAX;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*calcul rand1*/
     rand1=rand()/rand_m;
     /*test rand1*/
     while((rand1<=0)||(rand1>=1)) rand1=rand()/rand_m;

     /*calcul rand2*/
     rand2=rand()/rand_m;
     /*test rand2*/
     while((rand2<=0)||(rand2>=1)) rand2=rand()/rand_m;

     /*Calcul amplitude et phase*/
     ampl=(*sig)*sqrt(-log(rand1));
     phase=2*M_PI*rand2;

     /*Calcul y_c et y_q*/
     y_i[(*n)*l+i]=u_i[(*n)*l+i]+(*mean)+ampl*cos(phase);
     y_q[(*n)*l+i]=u_q[(*n)*l+i]+(*mean)+ampl*sin(phase);
   }
 }

 return;
}

void awgni_c(int *n,int *m,double *sig,double *mean,int *u,double *y)
{
 /*dclaration des variables*/
 int i,l;
 double rand1, rand2;
 double rand_m;
 double ampl, phase;

 /*rcupration de la valeur de RAND_MAX*/
 rand_m=RAND_MAX;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*calcul rand1*/
     rand1=rand()/rand_m;
     /*test rand1*/
     while((rand1<=0)||(rand1>=1)) rand1=rand()/rand_m;

     /*calcul rand2*/
     rand2=rand()/rand_m;
     /*test rand2*/
     while((rand2<=0)||(rand2>=1)) rand2=rand()/rand_m;

     /*Calcul amplitude et phase*/
     ampl=(*sig)*sqrt(2*-log(rand1));
     phase=2*M_PI*rand2;

     /*Calcul y*/
     y[(*n)*l+i]=(double)(u[(*n)*l+i])+(*mean)+ampl*cos(phase);
   }
 }

 return;
}

void awgniqi_c(int *n,int *m,double *sig,double *mean,\
              int *u_i,int *u_q,\
              double *y_i,double *y_q)
{
 /*dclaration des variables*/
 int i,l;
 double rand1, rand2;
 double rand_m;
 double ampl, phase;

 /*rcupration de la valeur de RAND_MAX*/
 rand_m=RAND_MAX;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*calcul rand1*/
     rand1=rand()/rand_m;
     /*test rand1*/
     while((rand1<=0)||(rand1>=1)) rand1=rand()/rand_m;

     /*calcul rand2*/
     rand2=rand()/rand_m;
     /*test rand2*/
     while((rand2<=0)||(rand2>=1)) rand2=rand()/rand_m;

     /*Calcul amplitude et phase*/
     ampl=(*sig)*sqrt(-log(rand1));
     phase=2*M_PI*rand2;

     /*Calcul y_i et y_q*/
     y_i[(*n)*l+i]=(double)(u_i[(*n)*l+i])+(*mean)+ampl*cos(phase);
     y_q[(*n)*l+i]=(double)(u_q[(*n)*l+i])+(*mean)+ampl*sin(phase);
   }
 }

 return;
}
