/* bin2decdd_c subroutine
 * binary vector to integer vector
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* bin2decdd_c routine de consersion binaire vers entier
 *
 * entre : double
 * sortie : double
 *
 * nu  : taille du vecteur d'entre
 * M   : nombre de bit par entier
 * msb : msb=1  msb en premier
 *       msb!=1 lsb en premier
 * sig : sig=1 entier sign
 * u   : adresse de dpart du vecteur d'entre
 * y   : adresse de dpart du vecteur de sortie
 *       (de taille nu/M)
 *
 */
void bin2decdd_c(int *nu,int *M,int *msb,int *sig,double *u,double *y)
{
 /*Dclaration des variables locales*/
 int u_i,y_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (double) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (double) y_i;
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       y[i] = (double) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       y[i] = (double) y_i;
     }
   }
 }
 return;
}

void bin2decdi_c(int *nu,int *M,int *msb,int *sig,double *u,long *y)
{
 /*Dclaration des variables locales*/
 int u_i,y_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (long) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (long) y_i;
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       y[i] = (long) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       y[i] = (long) y_i;
     }
   }
 }
 return;
}

void bin2decid_c(int *nu,int *M,int *msb,int *sig,long *u,double *y)
{
 /*Dclaration des variables locales*/
 int u_i,y_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (double) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (double) y_i;
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       y[i] = (double) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       y[i] = (double) y_i;
     }
   }
 }
 return;
}

void bin2decii_c(int *nu,int *M,int *msb,int *sig,long *u,long *y)
{
 /*Dclaration des variables locales*/
 int u_i,y_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (long) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       /* conversion en nombre sign (cc2) */
       y_i -= 2<<((*M)-2);
       y_i &= (2<<((*M)-1)) - 1;
       y_i -= 2<<((*M)-2);

       y[i] = (long) y_i;
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << (((*M)-1) - j));
       }

       y[i] = (long) y_i;
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<((*nu)/(*M));i++) {
       /* consersion double en int */
       y_i=0;

       /* place les valeurs des entiers sur la sortie */
       for (j=0;j<(*M);j++) {
         u_i = (int) u[i*(*M)+j];
         y_i = y_i + (u_i << j);
       }

       y[i] = (long) y_i;
     }
   }
 }
 return;
}
