/* cmplxadd_c subroutine
 * complex addition computation
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* cmplxadd_c routine de calcul d'addition-soustraction de vecteurs complexes
 *
 * n           : la taille des vecteurs
 * sig         : signe de l'opration
 * [z1_r;z1_i] : adresses de dpart du vecteur complexe 1
 * [z2_r;z2_i] : adresses de dpart du vecteur complexe 2
 * [y_r;y_i]   : adresses de dpart du vecteur complexe rsultat
 *
 * rmq : doit exister en version BLAS(!?)
 */

void cmplxadd_c(int *n,int *sig,double *z1_r,double *z1_i,double *z2_r,double *z2_i,double *y_r,double *y_i)
{
 /*dclaration des variables*/
 int i;

 /*ralise multiplication complex*/
 for(i=0;i<(*n);i++) {
  y_r[i]=z1_r[i]+(*sig)*z2_r[i];
  y_i[i]=z1_i[i]+(*sig)*z2_i[i];
 }
 return;
}

/* cmplxcopy_c routine qui copie deux tableaux dans deux autres
 * n     : longueur du tableau
 * u1    : tableau en entre (double)
 * u2    : tableau en entre (double)
 * y1    : tableau en sortie (double)
 * y2    : tableau en sortie (double)
 */

void cmplxcopy_c(int *n,double *u1,double *u2,double *y1,double *y2)
{
 /*Dclaration des variables*/
 int i;

 for(i=0;i<(*n);i++) {
   y1[i] = u1[i];
 }
 for(i=0;i<(*n);i++) {
   y2[i] = u2[i];
 }
 return;
}

/*
 * cmplxmult_c routine de calcul de multiplication de vecteurs complexes
 *
 * n           : la taille des vecteurs
 * [z1_r;z1_i] : adresses de dpart du vecteur complexe 1
 * [z2_r;z2_i] : adresses de dpart du vecteur complexe 2
 * [y_r;y_i]   : adresses de dpart du vecteur complexe rsultat
 *
 * rmq : doit exister en version BLAS(!?)
 */

void cmplxmult_c(int *n,double *z1_r,double *z1_i,double *z2_r,double *z2_i,double *y_r,double *y_i)
{
 /*dclaration des variables*/
 int i;

 /*ralise multiplication complex*/ 
 /*F2C(wmmul)(&z1_r[0],&z1_i[0],(n1=1,&n1),&z2_r[0],&z2_i[0],(m=1,&m),&y_r[0],&y_i[0],(m=1,&m),n,(m=1,&m),n);*/
 for(i=0;i<(*n);i++) {
  y_r[i]=z1_r[i]*z2_r[i]-z1_i[i]*z2_i[i];
  y_i[i]=z1_r[i]*z2_i[i]+z1_i[i]*z2_r[i];
 }
 return;
}
