/* dec2bindd_c subroutine
 * integer vector to binary vector
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* dec2bindd_c routine de convertion entier vers binaire
 *
 * entre : double
 * sortie : double
 *
 * nu  : taille du vecteur d'entre
 * M   : nombre de bit par entier
 * msb : msb=1  msb en premier
 *       msb!=1 lsb en premier
 * sig : sig=1 entier sign
 *       sig!=1 entier non sign
 * u   : adresse de dpart du vecteur d'entre
 * y   : adresse de dpart du vecteur de sortie
 *       (de taille M*nu)
 *
 */
void dec2bindd_c(int *nu,int *M,int *msb,int *sig,double *u,double *y)
{
 /*Dclaration des variables locales*/
 int u_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 return;
}

void dec2bindi_c(int *nu,int *M,int *msb,int *sig,double *u,long *y)
{
 /*Dclaration des variables locales*/
 int u_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* conversion double en int */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 return;
}

void dec2binid_c(int *nu,int *M,int *msb,int *sig,long *u,double *y)
{
 /*Dclaration des variables locales*/
 int u_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (double) (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 return;
}

void dec2binii_c(int *nu,int *M,int *msb,int *sig,long *u,long *y)
{
 /*Dclaration des variables locales*/
 int u_i,i,j;

 /* entier sign */
 if(*sig==1) {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* conversion en nombre non sign */
       u_i &= (2<<((*M)-1)) - 1;

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 /* entier non sign */
 else {
   /* msb en premier */
   if(*msb==1) {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+((*M)-1)-j] = (((1<<j) & u_i)>>j);
       }
     }
   }
   /* lsb en premier */
   else {
     for (i=0;i<(*nu);i++) {
       /* */
       u_i = (int) u[i];

       /* place les valeurs des bits sur la sortie */
       for (j=0;j<(*M);j++) {
         y[(i*(*M))+j] = (((1<<j) & u_i)>>j);
       }
     }
   }
 }
 return;
}



