/* demodqam_c subroutine
 * Quadrature Amplitude Modulation demodulator
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* demodqam_c routine de calcul d'un dmodulataur mQAM
 *
 * Entres :
 * n   : longueur des vecteurs d'entres
 * ns  : longueurs des mots binaires en nombre de bits (scalaire)
 * i_c : adresse de dpart du vecteur de la composante I
 * q_c : adresse de dpart du vecteur de la composante Q
 *
 * Sortie :
 * y : adresse de dpart du vecteur du symbole
 *
 * Dpendances:
 */
void demodqam_c(int *n,int *m,int*ns,double *i_c,double *q_c,double *y)
{
 /*dclaration des variables*/
 int i,l;
 int ng,nd;
 int d,g;

 /*Calcul des slecteurs binaires (c'est maladroit!!)*/
 nd=(1<<(*ns)/2)-1;
 ng=(1<<(*ns))-1-nd;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*rcupration des valeurs d'entre*/
     d=(int)i_c[(*n)*l+i];
     g=(int)q_c[(*n)*l+i];

     /*Calcul les nombres binaires droit et gauche*/
     d=(d+nd)/2;
     g=((g+nd)/2)<<((*ns)/2);

     /*Calcul du registre de sortie y[]*/
     y[(*n)*l+i]=d+g;
   }
 }
 return;
}

void demodqami_c(int *n,int *m,int*ns,int *i_c,int *q_c,int *y)
{
 /*dclaration des variables*/
 int i,l;
 int ng,nd;
 int d,g;

 /*Calcul des slecteurs binaires (c'est maladroit!!)*/
 nd=(1<<(*ns)/2)-1;
 ng=(1<<(*ns))-1-nd;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*rcupration des valeurs d'entre*/
     d=(int)i_c[(*n)*l+i];
     g=(int)q_c[(*n)*l+i];

     /*Calcul les nombres binaires droit et gauche*/
     d=(d+nd)/2;
     g=((g+nd)/2)<<((*ns)/2);

     /*Calcul du registre de sortie y[]*/
     y[(*n)*l+i]=d+g;
   }
 }
 return;
}

/* demodqamv_c routine de calcul d'un dmodulataur mQAM
 *
 * Entres :
 * n :longueur des vecteurs d'entres
 * m :longueurs des mots binaires en nombre de bits (vecteur)
 * i_c :adresse de dpart du vecteur de la composante I
 * q_c :adresse de dpart du vecteur de la composante Q
 *
 * Sortie :
 * y : adresse de dpart du vecteur du symbole
 *
 * Dpendances:
 */
void demodqamv_c(int *n,int *m,double *i_c,double *q_c,double *y)
{
 /*dclaration des variables*/
 int i;
 int ng,nd;
 int d,g;

 for(i=0;i<(*n);i++)
 {
  /*Rcupration du nombre d'tats*/
  /*n=block->ipar[i];*/

  /*Calcul des slecteurs binaires (c'est maladroit!!)*/
  nd=(1<<(m[i])/2)-1;
  ng=(1<<(m[i]))-1-nd;
  /*fprintf(stderr,"nd=%d, ng=%d\n", nd,ng);*/

  /*rcupration des valeurs d'entre*/
  d=(int)i_c[i];
  g=(int)q_c[i];

  /*Calcul les nombres binaires droit et gauche*/
  d=(d+nd)/2;
  g=((g+nd)/2)<<((m[i])/2);

  /*Calcul du registre de sortie y[]*/
  y[i]=d+g;
 }
 return;
}

