/* gold_c subroutine
 * Gold Sequence generator
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* gold_c routine de calcul dynamique de squences de gold
 * N     : longueur des registres 1 et 2
 * ny    : longueur du vecteur de sortie dsire
 * y     : adresse de dpart du vecteur rsultat y[0..ny-1]
 * reg1  : registre 1
 * reg2  : registre 2
 * coef1 : coef du registre 1
 * coef2 : coef du registre 2
 */

void gold_c(int *N, int *ny, double *y, int *reg1, int *reg2, int *coef1, int *coef2)
{
 /*Dclaration des variables*/
 int i,j;
 int j1=0;
 int j2=0;

 /* Dlivre la valeur du dernier bit sur le registre y[]
  * ralise un XOR  la sortie des deux registres
  */
 for(j=0;j<(*ny);j++)
  {
   if(((*reg1)&1)==((*reg2)&1)) y[j]=-1.; /*attention ici -1 et pas 0*/
   else y[j]=1.;

   /*Calcul de la nouvelle valeur du bit de poid fort des registres*/
   for(i=0;i<(*N);i++)
   {
    /*Test sur la valeur du coefficient i du registre 1*/
    if(((*coef1)&(1<<i))!=0)
    {
     /*Cas c_i=1*/
     if(i!=0)
     {
      /*Ralise opration XOR*/
      if((((*reg1)>>i)&1)==j1) j1=0;
      else j1=1;
     }
    else j1=(*reg1)&1;
    }
      
    /*Test sur la valeur du coefficient i du registre 2*/
    if(((*coef2)&(1<<i))!=0)
    {
     /*Cas c_i=1*/
     if(i!=0)
     { 
      /*Ralise opration XOR*/
      if((((*reg2)>>i)&1)==j2) j2=0;
      else j2=1;
     }
     else j2=(*reg2)&1;
    }
   }
   /*Dcale le registre 1 et 2 de 1 bit vers la droite*/
   (*reg1) = (*reg1)>>1;
   (*reg2) = (*reg2)>>1;

   /*Ajoute le bit de poid fort*/
   (*reg1) += (j1<<(*N-1));
   (*reg2) +=  (j2<<(*N-1));
  } 
return;
}

void goldi_c(int *N, int *ny, int *y, int *reg1, int *reg2, int *coef1, int *coef2)
{
 /*Dclaration des variables*/
 int i,j;
 int j1=0;
 int j2=0;

 /* Dlivre la valeur du dernier bit sur le registre y[]
  * ralise un XOR  la sortie des deux registres
  */
 for(j=0;j<(*ny);j++)
  {
   if(((*reg1)&1)==((*reg2)&1)) y[j]=-1; /*attention ici -1 et pas 0*/
   else y[j]=1;

   /*Calcul de la nouvelle valeur du bit de poid fort des registres*/
   for(i=0;i<(*N);i++)
   {
    /*Test sur la valeur du coefficient i du registre 1*/
    if(((*coef1)&(1<<i))!=0)
    {
     /*Cas c_i=1*/
     if(i!=0)
     {
      /*Ralise opration XOR*/
      if((((*reg1)>>i)&1)==j1) j1=0;
      else j1=1;
     }
    else j1=(*reg1)&1;
    }

    /*Test sur la valeur du coefficient i du registre 2*/
    if(((*coef2)&(1<<i))!=0)
    {
     /*Cas c_i=1*/
     if(i!=0)
     { 
      /*Ralise opration XOR*/
      if((((*reg2)>>i)&1)==j2) j2=0;
      else j2=1;
     }
     else j2=(*reg2)&1;
    }
   }
   /*Dcale le registre 1 et 2 de 1 bit vers la droite*/
   (*reg1) = (*reg1)>>1;
   (*reg2) = (*reg2)>>1;

   /*Ajoute le bit de poid fort*/
   (*reg1) += (j1<<(*N-1));
   (*reg2) +=  (j2<<(*N-1));
  } 
return;
}
