/* modnum_fft.h header for modnum_fft
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifndef MODNUM_FFT_H
#define MODNUM_FFT_H

#include <stdlib.h>

#if WITH_FFTW3
#include <fftw3.h>
#else
typedef struct {
  int n;
  int is;
  int os;
} fftw_iodim;
typedef int fftw_plan;
fftw_plan fftw_plan_guru_split_dft(int rank, const fftw_iodim *dims,
                                   int howmany_rank, const fftw_iodim *howmany_dims,
                                   double *ri, double *ii, double *ro, double *io,
                                   unsigned flags) {return 1;}
void fftw_execute_split_dft(int p,double * r_i,double * i_i,double * r_o,double * i_o) {}
void fftw_destroy_plan(int p) {}
#define FFTW_ESTIMATE 1
#endif

#if WITH_SCIFFT
#include "machine.h"
#else
void fft842(int *k,int *n,double *r_i,double *i_i,int *ierr) {}
void dfft2(double *r_i,double *i_i,int *i,int *n,int *j,int *k,int *ierr,double *work,int *n_work) {}
#define C2F(name) name
#endif

typedef struct guru_dim_st {
  int rank;
  fftw_iodim *dims;
  int howmany_rank;
  fftw_iodim *howmany_dims;
} guru_dim_struct;

typedef struct fftw_plan_st {
  fftw_plan p;
  guru_dim_struct gdim;
  unsigned flags;
} fftw_plan_struct;

fftw_plan get_fftw_plan(guru_dim_struct *gdim,
                        double *ri, double *ii,
                        double *ro, double *io,
                        unsigned flags, int isn);

int free_fftw_plan(fftw_plan_struct *modnum_plan);

int check_guru_dims(guru_dim_struct *gdim1,
                    guru_dim_struct *gdim2);

static fftw_plan_struct forward_plan =
{
  0,             /* fftw_plan p */
  {0,0,0,0},     /* guru_dim_struct gdim */
  FFTW_ESTIMATE  /* unsigned flags */
};

static fftw_plan_struct backward_plan =
{
  0,             /* fftw_plan p */
  {0,0,0,0},     /* guru_dim_struct gdim */
  FFTW_ESTIMATE  /* unsigned flags */
};

static unsigned cur_fftw_flags=FFTW_ESTIMATE;

int check_guru_dims(guru_dim_struct *gdim1, guru_dim_struct *gdim2)
{
  int i;

  if( (gdim1->rank==gdim2->rank) &&
      (gdim1->howmany_rank==gdim2->howmany_rank)) {
    for(i=0;i<gdim1->rank;i++) {
      if (gdim1->dims[i].n  != gdim2->dims[i].n)  return(0);
      if (gdim1->dims[i].is != gdim2->dims[i].is) return(0);
      if (gdim1->dims[i].os != gdim2->dims[i].os) return(0);
    }
    for(i=0;i<gdim1->howmany_rank;i++) {
      if (gdim1->howmany_dims[i].n  != gdim2->howmany_dims[i].n)  return(0);
      if (gdim1->howmany_dims[i].is != gdim2->howmany_dims[i].is) return(0);
      if (gdim1->howmany_dims[i].os != gdim2->howmany_dims[i].os) return(0);
    }
    return(1);
  }
  else return(0);
}

int free_fftw_plan(fftw_plan_struct *modnum_plan)
{
  if(modnum_plan->p != 0) {
    fftw_destroy_plan(modnum_plan->p);
    modnum_plan->p = 0;
  }
  if(modnum_plan->gdim.rank != 0) {
    modnum_plan->gdim.rank = 0;
    free(modnum_plan->gdim.dims);
    modnum_plan->gdim.dims = 0;
  }
  if(modnum_plan->gdim.howmany_rank != 0) {
    modnum_plan->gdim.howmany_rank = 0;
    free(modnum_plan->gdim.howmany_dims);
    modnum_plan->gdim.howmany_dims = 0;
  }
  return(1);
}

fftw_plan get_fftw_plan(guru_dim_struct *gdim,
                        double *ri, double *ii,
                        double *ro, double *io,
                        unsigned flags, int isn)
{
  fftw_plan_struct *modnum_plan;
  int i;

  if (isn==-1) modnum_plan = &backward_plan;
  else modnum_plan = &forward_plan;

  if ( (!(check_guru_dims(&(modnum_plan->gdim), gdim))) ||
       (modnum_plan->flags != cur_fftw_flags) ) {

    free_fftw_plan(modnum_plan);

    if (gdim->rank != 0) {
      modnum_plan->gdim.rank = gdim->rank;
      if ((modnum_plan->gdim.dims = \
              (fftw_iodim *)malloc(sizeof(fftw_iodim)*gdim->rank))==0) {
       return(0);
      }
      for (i=0;i<gdim->rank;i++) {
        modnum_plan->gdim.dims[i].n  = gdim->dims[i].n;
        modnum_plan->gdim.dims[i].is = gdim->dims[i].is;
        modnum_plan->gdim.dims[i].os = gdim->dims[i].os;
      }
    }

    if (gdim->howmany_rank != 0) {
      modnum_plan->gdim.howmany_rank = gdim->howmany_rank;
      if ((modnum_plan->gdim.howmany_dims = \
             (fftw_iodim *)malloc(sizeof(fftw_iodim)*gdim->howmany_rank))==0) {
       free(modnum_plan->gdim.dims);
       return(0);
      }
      for (i=0;i<gdim->howmany_rank;i++) {
        modnum_plan->gdim.howmany_dims[i].n  = gdim->howmany_dims[i].n;
        modnum_plan->gdim.howmany_dims[i].is = gdim->howmany_dims[i].is;
        modnum_plan->gdim.howmany_dims[i].os = gdim->howmany_dims[i].os;
      }
    }

    modnum_plan->flags = cur_fftw_flags;

    modnum_plan->p = fftw_plan_guru_split_dft(modnum_plan->gdim.rank,
                                              modnum_plan->gdim.dims,
                                              modnum_plan->gdim.howmany_rank,
                                              modnum_plan->gdim.howmany_dims,
                                              ri, ii, ro, io,
                                              modnum_plan->flags);
  }
  return(modnum_plan->p);
}

#endif /* MODNUM_FFT_H */
