/* modqam_c subroutine
 * Quadrature Amplitude Modulation Modulator
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* modqam_c routine de calcul d'un modulataur mQAM
 *
 * Entres :
 * n  : dimension 1 des matrices d'entre/sorties (scalaire)
 * m  : dimension 2 des matrices d'entre/sorties (scalaire)
 * ns : longueurs des mots binaires en nombre de bits (scalaire)
 * u  : adresse de dpart du vecteur du symbole en entre
 *
 * Sortie :
 * i_c : adresse de dpart du vecteur de la composante I
 * q_c : adresse de dpart du vecteur de la composante Q
 *
 * Dpendances:
 */

void modqam_c(int *n,int *m,int *ns,double *u,double *i_c,double *q_c)
{
 /*dclaration des variables compteurs*/
 int i,j,l;
 int ng,nd;

 /*Calcul des slecteurs binaires (c'est maladroit!!)*/
 nd=(1<<(*ns)/2)-1;
 ng=(1<<(*ns))-1-nd;

 for(l=0;l<(*m);l++) {
   for(j=0;j<(*n);j++) {
     /*rcupration de la valeur du port d'entre*/
     i=(int)u[(*n)*l+j];

     /*Calcul de la valeur de I et de Q*/
     i_c[(*n)*l+j]=((i&nd)*2)-nd;
     q_c[(*n)*l+j]=(((i&ng)>>((*ns)/2))*2)-nd;
   }
 }

 return;
}

void modqami_c(int *n,int *m,int *ns,int *u,int *i_c,int *q_c)
{
 /*dclaration des variables compteurs*/
 int i,j,l;
 int ng,nd;

 /*Calcul des slecteurs binaires (c'est maladroit!!)*/
 nd=(1<<(*ns)/2)-1;
 ng=(1<<(*ns))-1-nd;

 for(l=0;l<(*m);l++) {
   for(j=0;j<(*n);j++) {
     /*rcupration de la valeur du port d'entre*/
     i=(int)u[(*n)*l+j];

     /*Calcul de la valeur de I et de Q*/
     i_c[(*n)*l+j]=((i&nd)*2)-nd;
     q_c[(*n)*l+j]=(((i&ng)>>((*ns)/2))*2)-nd;
   }
 }

 return;
}

/* modqamv_c routine de calcul d'un modulataur mQAM
 *
 * Entres :
 * n :longueur du vecteur d'entre
 * m :longueurs des mots binaires en nombre de bits (vecteur)
 * u :adresse de dpart du vecteur du symbole en entre
 *
 * Sortie :
 * i_c : adresse de dpart du vecteur de la composante I
 * q_c : adresse de dpart du vecteur de la composante Q
 *
 * Dpendances:
 */
void modqamv_c(int *n,int *m,double *u,double *i_c,double *q_c)
{
 /*dclaration des variables compteurs*/
 int i,j;
 int ng,nd;

 for(j=0;j<(*n);j++) {
   /*rcupration du nombre de bits associ  chaque axes*/
   /*n = block->ipar[j];*/

   /*Calcul des slecteurs binaires (c'est maladroit!!)*/
   nd=(1<<(m[j])/2)-1;
   ng=(1<<(m[j]))-1-nd;

   /*rcupration de la valeur du port d'entre*/
   i=(int)u[j];

   /*Calcul de la valeur de I et de Q*/
   i_c[j]=((i&nd)*2)-nd;
   q_c[j]=(((i&ng)>>((m[j])/2))*2)-nd;
 }
 return;
}
