/* sousecht_c subroutine
 * Down-Sampling Computation
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* sousecht_c routine de calcul de sous-chantillonnage en temporel
 *
 * Entres :
 * n     : dimension 1 de la matrice d'entre (scalaire)
 * m     : dimension 2 de la matrice d'entre (scalaire)
 * nech   : facteur de sous-chantillonnage
 * init_c : valeur initiale du compteur (doit tre entre 0 et nech-1)
 * u      : matrice d'entre  sous-chantillonner de taille n,m
 *
 * Sorties :
 * y      : matrice de sortie de taille int(n/nech),m
 *
 * Entre/Sortie :
 * init_c : valeur initiale du compteur (vecteur de taille m)
 *
 */

void sousecht_c(int *n,int *m,int *nech,int *init_c,double *u,double *y)
{
 /*Dclaration des variables compteurs*/
 int i,j,l;
 int count;

 for(l=0;l<(*m);l++) {
   /*Rcupre valeur initiale du compteur*/
   count=init_c[l];
   /*raz j*/
   j=0;
   /*Pour tous les chantillons des vecteurs d'entre*/
   for(i=0;i<(*n);i++) {
     /*if(i==(count-1))*/
     if(i==(count-1)) {
       y[l*((*n)/(*nech))+j]=u[l*(*n)+i];
       count += (*nech);
       j++;
     }
   }
   init_c[l]=Abs(count-(*n));
 }
 return;
}

void sousechti_c(int *n,int *m,int *nech,int *init_c,int *u,int *y)
{
 /*Dclaration des variables compteurs*/
 int i,j,l;
 int count;

 for(l=0;l<(*m);l++) {
   /*Rcupre valeur initiale du compteur*/
   count=init_c[l];
   /*raz j*/
   j=0;
   /*Pour tous les chantillons des vecteurs d'entre*/
   for(i=0;i<(*n);i++) {
     /*if(i==(count-1))*/
     if(i==(count-1)) {
       y[l*((*n)/(*nech))+j]=u[l*(*n)+i];
       count += (*nech);
       j++;
     }
   }
   init_c[l]=Abs(count-(*n));
 }
 return;
}

void sousechty_c(int *n,int *m,int *nech,int *init_c,double *u,double *y)
{
 /*Dclaration des variables compteurs*/
 int i,j,l;
 int count;

 for(l=0;l<(*m);l++) {
   /*Rcupre valeur initiale du compteur*/
   count=init_c[l];
   /*raz j*/
   j=0;
   /*Pour tous les chantillons des vecteurs d'entre*/
   for(i=0;i<(*n);i++) {
     /*if(i==(count-1))*/
     if(i==(count-1)) {
       y[l*((*n)/(*nech))+j]=u[l*(*n)+i];
       count += (*nech);
       j++;
     }
   }
 }
 return;
}
